#ifndef	   REF_COUNT_PTR_H_INCLUDED
#define	   REF_COUNT_PTR_H_INCLUDED

// Author:		H. Shimora
// Last-Modified:	Dec 29 1998
// Version:		0.20

///-----------------------------------------------
/// Change Log:
///-----------------------------------------------
// version 0.10  Dec 28 1998    base version.
//
// version 0.20  Dec 29 1998    separate counter and pointer to entity.
//                              memory allocation reduced.
//
//

#ifdef HAVE_CONFIG_H
# include  "config.h"
#endif


#include  <cstddef>
#include  <cassert>

template<class T>
class ref_count_ptr
{
private:
	typedef  unsigned int  REF_COUNTER_TYPE;

	T *				entity;
	mutable	REF_COUNTER_TYPE *	reference_counter;

private:
	void	release() // throw()
	{
		if ( entity )
		{
			if ( reference_counter == NULL )
			{
				delete entity;
			}
			else
			{

				if ( (-- (*reference_counter)) == 0 )
				{
					delete entity;
					delete reference_counter;
					entity            = NULL;
					reference_counter = NULL;
				}
			}
		}
	}

	void	substitute( const ref_count_ptr &  ptr ) // throw()
	{
		if ( (entity = ptr.entity) )
		{
			if ( ptr.reference_counter == NULL )
			{
				reference_counter
					= new REF_COUNTER_TYPE( 2 );
				ptr.reference_counter = reference_counter;
			}
			else
			{
				reference_counter = ptr.reference_counter;
				(*reference_counter) ++;
			}
		}
	}

	template<class U>
	void	substitute( const ref_count_ptr<U> &  ptr ) // throw()
	{
		if ( (entity = ptr.entity) )
		{
			if ( ptr.reference_counter == NULL )
			{
				reference_counter
					= new REF_COUNTER_TYPE( 2 );

				ptr.reference_counter = reference_counter;
			}
			else
			{
				reference_counter = ptr.reference_counter;
				(*reference_counter) ++;
			}
		}
	}

public:
	ref_count_ptr() throw()
		: entity( NULL ) ,
		  reference_counter( NULL )
	{
	}

	explicit  ref_count_ptr( const ref_count_ptr &  ptr ) throw()
		: entity( NULL ) ,
		  reference_counter( NULL )
	{
		substitute( ptr );
	}

	template<class U>
	explicit ref_count_ptr( const ref_count_ptr<U> &  ptr ) throw()
		: entity(NULL) ,
		  reference_counter( NULL )
	{
		substitute( ptr );
	}

	explicit  ref_count_ptr( const T * raw_pointer ) throw()
		: entity( const_cast<T*>(raw_pointer) ) ,
		  reference_counter( NULL )
	{
	}

	~ref_count_ptr()
	{
		release();
	}

	ref_count_ptr &  operator= ( const ref_count_ptr &  ptr ) throw()
	{
#if 0
		if ( &ptr == this )
#else
		if ( ptr.entity == this -> entity )
#endif
		{
			return( *this );
		}
		else
		{
			release();

			substitute( ptr );

			return( *this );
		}
	}

	template<class U>
	ref_count_ptr &  operator= ( const ref_count_ptr<U> &  ptr ) throw()
	{
#if 0
		if ( &ptr == this )
#else
		if ( ptr.entity == this -> entity )
#endif
		{
			return( *this );
		}
		else
		{
			release();

			substitute( ptr );

			return( *this );
		}
	}

	ref_count_ptr &  operator= ( const T *  raw_pointer ) throw()
	{
		release();

		entity            = const_cast<T*>(raw_pointer);
		reference_counter = NULL;

		return( *this );
	}

	T &  operator* () const throw()
	{
		return( *entity );
	}

	T *operator-> () const throw()
	{
		return( entity );
	}

	operator bool () const throw()
	{
		return( entity != NULL );
	}

	T *  get() const throw()
	{
		return( entity );
	}
};


#endif	/* REF_COUNT_PTR_H_INCLUDED */
