#include  "command_queue.h"

using namespace std;

//
// Command_Queue
//

Command_Queue::Command_Queue() throw()
{
}

Command_Queue::Command_Queue( const Soccer_Composite_Command &  com ) throw()
{
	this -> push( com );
}

Command_Queue::Command_Queue( const ref_count_ptr<Soccer_Action> &  act )
	throw()
{
	this -> push( act );
}

Command_Queue::Command_Queue( Soccer_Action *  act ) throw()
{
	this -> push( act );
}

Command_Queue::~Command_Queue() throw()
{
}


Command_Queue&  Command_Queue::push( const Soccer_Composite_Command &  com )
	throw()
{
	Command_or_Action	com_or_act;

	com_or_act.is_command = true;
	com_or_act.command = com;

	q.push( com_or_act );

	return( *this );
}

Command_Queue&  Command_Queue::push( const Soccer_Composite_Command &  com_1 ,
				     const Soccer_Composite_Command &  com_2 )
	throw()
{
	try
	{
		this -> push( com_1 + com_2 );

	} catch(...){}

	return( *this );
}


Command_Queue&  Command_Queue::push(
			    const ref_count_ptr<Soccer_Action> &  act ) throw()
{
#if 0
	if ( ! (act -> finished()) )
#endif
	{
		Command_or_Action	com_or_act;

		com_or_act.is_command = false;
		com_or_act.action = act;

		q.push( com_or_act );
	}

	return( *this );
}


Command_Queue&  Command_Queue::push( Soccer_Action *  act ) throw()
{
	ref_count_ptr<Soccer_Action>	act_ref( act );

	this -> push( act_ref );

	return( *this );
}


Soccer_Composite_Command  Command_Queue::pop() throw( exception )
{
	this -> remove_finished_action();

	if ( q.empty() )
	{
		throw exception();
	}

	Soccer_Composite_Command	ret;
	Command_or_Action		com_or_act;

	if ( q.front().is_command )
	{
		ret = q.front().command;
		q.pop();
	}
	else
	{
		ret = (q.front().action) -> get_command();
		if ( (q.front().action) -> finished() )
		{
			q.pop();
		}
	}

	return( ret );
}


bool   Command_Queue::empty() const throw()
{
	this -> remove_finished_action();

	return( q.empty() );
}


// protected
void   Command_Queue::remove_finished_action() const throw()
{
	for(;;)
	{
		if ( q.empty()
		  || (q.front().is_command
		     && q.front().command.base_type()
			  != Soccer_Command::Illegal
		     && q.front().command.base_type()
			  != Soccer_Command::No_Command)
		  || (! q.front().is_command
		     && ! (q.front().action -> finished())) )
		{
			return;
		}
		else
		{
			(void)(q.pop());
		}
	}
}


void   Command_Queue::clear() throw()
{
	while( ! q.empty() )
	{
		q.pop();
	}
}


Command_Queue::operator bool() const
{
	return( ! (this -> empty()) );
}




//
// Exclusive_Queue_Controler
//
#include  <iostream>

Exclusive_Queue_Controler::Exclusive_Queue_Controler()
{
}

Exclusive_Queue_Controler::~Exclusive_Queue_Controler()
{
}

Command_Queue *  Exclusive_Queue_Controler::register_queue
			( Command_Queue *  q ) throw()
{
	if ( queue_set.find( q ) != queue_set.end() )
	{
		cerr << "Exclusive queue registered" << endl;
	}

	queue_set.insert( q );

	return( q );
}


Command_Queue &  Exclusive_Queue_Controler::push(
				 Command_Queue &  q ,
				 const Soccer_Composite_Command & com ) throw()
{
	this -> clear_other_queue( q );

	q.push( com );

	return( q );
}


Command_Queue &  Exclusive_Queue_Controler::push(
				 Command_Queue &  q ,
				 const Soccer_Composite_Command & com_1 ,
				 const Soccer_Composite_Command & com_2 )
	throw()
{
	this -> clear_other_queue( q );

	q.push( com_1 , com_2 );

	return( q );
}


Command_Queue &  Exclusive_Queue_Controler::push
			( Command_Queue &  q ,
			  const ref_count_ptr<Soccer_Action> &  act ) throw()
{
	this -> clear_other_queue( q );

	q.push( act );

	return( q );
}

Command_Queue &  Exclusive_Queue_Controler::push( Command_Queue &  q ,
						  Soccer_Action *  act )
	throw()
{
	this -> clear_other_queue( q );

	q.push( act );

	return( q );
}


const  Command_Queue &  Exclusive_Queue_Controler::clear_other_queue
				( const Command_Queue &  q ) throw()
{
	bool	found = false;

	for( set<Command_Queue *>::iterator  i = queue_set.begin()  ;
	     i != queue_set.end() ;  i ++ )
	{
		if ( const_cast<const Command_Queue *>( *i ) == &q )
		{
			found = true;
		}
		else
		{
			(*i) -> clear();
		}
	}

	if ( ! found )
	{
		cerr << "no such queue" << endl;
	}

	return( q );
}


void   Exclusive_Queue_Controler::clear_all_queue() throw()
{
	for( set<Command_Queue *>::iterator  i = queue_set.begin()  ;
	     i != queue_set.end() ;  i ++ )
	{
		(*i) -> clear();
	}
}
