#include  "soccer_interface.h"
#include  "sserver_connection_with_game_info.h"
#include  "sserver_param.h"
#include  "soccer_option_analyzer.h"
#include  "soccer_composite_command.h"
#include  "ref_count_ptr.h"
#include  "time_stamp.h"

// XXX
#include  "autogen_act_savior.h"
#include  "autogen_act_test.h"

#include  <cstdlib>
#include  <iostream>
#include  <cstdio>
#include  <cstddef>
#include  <csignal>
#include  <stdexcept>

#ifdef HAVE_CONFIG_H
  #include  "config.h"
#endif
#ifndef RETSIGTYPE
  typedef void RETSIGTYPE;
#endif

using namespace std;


// for signal
static	bool					sig_serv_initialized = false;
static	SServer_Connection_with_Game_Info *	sig_serv = NULL;
static	RETSIGTYPE	signal_handler( int );

static
RETSIGTYPE   signal_handler( int )
{
	if ( sig_serv_initialized )
	{
		sig_serv -> send_bye();
	}

	exit( 0 );
}


int    main( int  argc ,  char  *argv[] )
{
	ios::sync_with_stdio();

#if 0
#ifdef RELEASE
	Soccer_Option_Analyzer	opt( argc , argv , "savior" );
#else
	Soccer_Option_Analyzer	opt( argc , argv ,
				     "savior" , "localhost" , 6100 );
#endif
#else
#ifdef RELEASE
	Soccer_Option_Analyzer	opt( argc , argv , "Zeng99" );
#else
	Soccer_Option_Analyzer	opt( argc , argv ,
				     "Zeng99" , "localhost" , 6100 );
#endif
#endif

	if ( ! opt )
	{
		opt.usage();
		exit( 1 );
	}

	try
	{
		SServer_Param		sserver_param;

		SServer_Connection_with_Game_Info	server(
				opt.hostname() ,
				opt.port_number() ,
				true ,  // use radian for send
				true ,  // use radian for recv
				static_cast<long>
				( sserver_param.RECV_STEP() * 1000 * 1.1 ) ,
#ifdef RELEASE
				SServer_Socket::DEFAULT_MAX_RESPONCE_TIME
#else
				0
#endif
				);

		if ( ! server )
		{
			cerr << "Illegal server connenction" << endl;
			exit( 1 );
		}

		sig_serv = &server;
		sig_serv_initialized = true;
		if ( signal( SIGHUP  , signal_handler ) == SIG_ERR
		  || signal( SIGINT  , signal_handler ) == SIG_ERR
		  || signal( SIGQUIT , signal_handler ) == SIG_ERR )
		{
			perror( "signal" );
			exit( 1 );
		}

		Soccer_Interface	sif( server , sserver_param );

		Game_Info	game_info;
		if ( opt.reconnect() == false )
		{
			if ( sif.send_init( opt.teamname() , true ,
					    &game_info , 5 , 0 ) == -1 )
			{
				cerr << "init command failed." << endl;
				exit(1);
			}
		}
		else
		{
			if ( sif.send_reconnect(
					opt.teamname() ,
					opt.reconnect_player_number() ,
					&game_info , true ) == -1 )
			{
				cerr << "reconnect command failed." << endl;
				exit(1);
			}
		}

		ref_count_ptr<Field_Recog_Abstract>	field_recog(
			new Field_Recog( game_info , sserver_param ) );

		sif.register_field_recog( field_recog );

#if 1
		sif.set_automatic_change_view
		      ( Soccer_Interface::Auto_Change_View_Type::Narrow_High );
#endif

#if 1
		sif.send_say( "Zeng99-GK" , true );
#endif

#if 1
		ref_count_ptr<Soccer_Action>  act(new Act_Savior(field_recog));
#else
		ref_count_ptr<Soccer_Action>  act(new Act_Test(field_recog));
#endif

		for(;;)
		{
			cout << "wait_for_next_turn" << endl;

			sif.wait_for_next_turn( 100 * 1000 /* usec */ );

			if ( sif.game_is_over() )
			{
				cout << "Game is Over" << endl;
				sif.send_bye();
				break;
			}
			else if ( sif.server_no_response() )
			{
				cout << "Server No Response" << endl;
				sif.send_bye();
				break;
			}

#if 1
			Time_Stamp	t0;
			t0.set_current_time();
#endif

			cout << "act -> get_command()" << endl;

			Soccer_Composite_Command	command;
			command = act -> get_command();

			if ( command.base_type() == Soccer_Command::Illegal )
			{
				cerr << "Illegal Command" << endl;
			}
			else if( command.base_type()
				     == Soccer_Command::No_Command )
			{
				cout << "No Command" << endl;
			}

			sif.send_composite_command( command , false );

#if 1
			Time_Stamp	t1;
			t1.set_current_time();

			cout << "thinking time = "
			     << (t1 - t0).msec_real() << "msec" << endl;
#endif
			cout << endl << endl << endl << endl;
		}
	}
	catch( const exception &  e )
	{
		cerr << "exception caught!! [" << e.what() << "]" << endl;
		return( 1 );
	}
	catch( ... )
	{
		cerr << "exception caught!!" << endl;
		return( 1 );
	}

	return( 0 );
}
