#ifndef	   FIELD_RECOG_H_INCLUDED
#define	   FIELD_RECOG_H_INCLUDED

#include  "field_recog_abstract.h"
#include  "field_state.h"
#include  "information_from_server.h"
#include  "game_info.h"
#include  "s_basic.h"
#include  "sserver_turn.h"
#include  <vector>
#include  <string>
#include  <stdexcept>

class  Field_Recog : public Field_Recog_Abstract
{
public:
	class  Field_Info_Exception{};


protected:
	Field_State		state;
	double			my_neck_angle;

	View_Angle		v_angle;
	View_Quality		v_quality;

	SServer_Turn		strict_ball_coordinate_and_vector_info_time;
	SServer_Turn		strict_ball_coordinate_info_time;
	SServer_Turn		strict_my_coordinate_and_angle_info_time;

	int			sight_count;
	int			ball_time_count;
	std::vector<int>	teammate_time_count;
	std::vector<int>	opponent_time_count;

	Sight_Info_from_Server	latest_sight;
	SServer_Turn		latest_sight_time;

	Body_Info_from_Server	body_info;
	Body_Info_from_Server	old_body_info;
	Body_Info_from_Server	latest_body;
	SServer_Turn		latest_body_time;

	Whistle_Info_from_Server	last_judgement;
	int				last_judgement_t;
	int				play_mode_start_t;

	D2_Vector		imaginary_ball_coordinate;
	D2_Vector		imaginary_ball_vector;

static	Debug_Info_from_Server	latest_debug;
static	int			latest_debug_time;

protected:
	void	register_sight_info( const Sight_Info_from_Server &  sight ,
				     int  info_time ) throw();

	void	register_body_info(
			   const Body_Info_from_Server &  body ,
			   int  info_time ) throw();

	void	register_whistle_info(
			   const Whistle_Info_from_Server &  whistle ,
			   int  info_time ) throw();

	void	register_debug_info(
			   const Debug_Info_from_Server &  debug ,
			   int  info_time ) throw();
public:
	 Field_Recog( const Game_Info &  g_info ,
		      const SServer_Param  &  param );

virtual	~Field_Recog();

	//
	// protocol between Soccer_Interface and Field_Recog
	//
virtual	void	register_info( const Information_from_Server &  info ) throw();

virtual	void	next_turn( const Soccer_Command &  prev ,
			   bool  last_command_accepted = true ) throw();

virtual	void	turn_neck( double  moment ) throw();

virtual	const SServer_Param &		sserver_param() const throw();
virtual	const Game_Info &		get_game_info() const throw();

virtual	SServer_Turn			current_time() const throw();

virtual	const Sight_Info_from_Server &	get_latest_sight_info() const throw();
virtual	SServer_Turn			get_latest_sight_time() const throw();

virtual	const Body_Info_from_Server &	get_latest_body_info() const throw();
virtual	SServer_Turn			get_latest_body_time() const throw();

virtual	const Field_State &		get_field_state() const throw();




virtual	Play_Mode			play_mode() const throw();
virtual Whistle_Info_from_Server	play_mode_reason() const throw();
virtual	int				play_mode_start_time() const throw();
virtual	int				last_judgement_time() const throw();

virtual	int		my_player_number() const throw();

virtual	View_Angle	view_angle() const throw();
virtual	View_Quality	view_quality() const throw();

virtual	SServer_Turn	get_strict_ball_coordinate_and_vector_info_time()
								const throw();
virtual	SServer_Turn	get_strict_ball_coordinate_info_time()  const throw();
virtual	SServer_Turn	get_strict_my_coordinate_and_angle_info_time()
								const throw();

virtual	int		strict_ball_lose_turn( int  max_turn = 10 )
								const throw();


	//
	// Encode
	//
virtual	operator std::string() const;




	//
	// some short cuts
	//
virtual	D2_Vector	get_my_coordinate()   const /*throw(std::exception)*/;
virtual	double		get_my_body_angle()   const /*throw(std::exception)*/;
virtual	double		get_my_face_angle()   const /*throw(std::exception)*/;
virtual	D2_Vector	get_ball_coordinate() const /*throw(std::exception)*/;
virtual	D2_Vector	get_ball_relative()   const /*throw(std::exception)*/;
virtual	D2_Vector	get_ball_vector()     const /*throw(std::exception)*/;
virtual	D2_Vector	get_player_coordinate
				( const SObject_Player_Identifier & )
					const /*throw(std::exception)*/;
virtual	D2_Vector	get_player_coordinate( S_Side , int )
					const /*throw(std::exception)*/;
virtual	D2_Vector	get_teammate_coordinate( int )
					const /*throw(std::exception)*/;
virtual	D2_Vector	get_opponent_coordinate( int )
					const /*throw(std::exception)*/;

virtual	double		get_my_stamina() const throw();
virtual	double		get_my_speed()   const throw();

virtual	D2_Vector	get_imaginary_ball_coordinate() const throw();
virtual	D2_Vector	get_imaginary_ball_vector()     const throw();


static	const Debug_Info_from_Server &	server_debug_info() throw();
static	int				server_debug_info_time() throw();
};


#endif	/* FIELD_RECOG_H_INCLUDED */
