#include  "field_state.h"
#include  <stdexcept>
#include  <cassert>

using namespace std;

Field_State::Field_State( int  my_player_num ,
			  size_t  n_teammate ,
			  size_t  n_opponent ) throw()
	: my_player_number( my_player_num ) ,
	  teammate_position( n_teammate ) , opponent_position( n_opponent ) ,
	  ball_position()
{
	assert( 1 <= my_player_number
		 &&  my_player_number
			<= static_cast<int>(teammate_position.size()) );

	for ( size_t  i = 0  ;  i < teammate_position.size()  ;  i ++ )
	{
		teammate_position[i].have_info       = false;
		teammate_position[i].have_body_angle = false;
		teammate_position[i].body_angle      = 0.0;
		teammate_position[i].have_face_angle = false;
		teammate_position[i].face_angle      = 0.0;
	}

	for ( size_t  i = 0  ;  i < opponent_position.size()  ;  i ++ )
	{
		opponent_position[i].have_info       = false;
		opponent_position[i].have_body_angle = false;
		opponent_position[i].body_angle      = 0.0;
		opponent_position[i].have_face_angle = false;
		opponent_position[i].face_angle      = 0.0;
	}

	ball_position.have_info   = false;
	ball_position.have_vector = false;
}

Field_State::Field_State( const string &  str )
{
	// XXX
	(void)str;
}

Field_State::~Field_State()
{
}




//
// Set Information
//
void   Field_State::set_ball_info( const D2_Region &  region ) throw()
{
	ball_position.region    = region;
	ball_position.have_info = true;
}

void   Field_State::set_ball_info( const D2_Region &  region ,
				   const D2_Region &  vec ) throw()
{
	set_ball_info( region );
	ball_position.vec         = vec;
	ball_position.have_vector = true;
}


void   Field_State::set_player_info( S_Side  side , int  player_number ,
				     const D2_Region &  region ) throw()
{
	assert( side == S_Side::Our_Side || side == S_Side::Opponent_Side );

	if ( side == S_Side::Our_Side )
	{
		set_teammate_info( player_number , region );
	}
	else
	{
		set_opponent_info( player_number , region );
	}
}

void   Field_State::set_player_info( S_Side  side , int  player_number ,
				     const D2_Region &  region ,
				     double  angle ) throw()
{
	if ( side == S_Side::Our_Side )
	{
		set_teammate_info( player_number , region , angle );
	}
	else
	{
		set_opponent_info( player_number , region , angle );
	}
}

void   Field_State::set_player_info( S_Side  side ,  int  player_number ,
				     const D2_Region &  region ,
				     double  body_angle ,
				     double  face_angle ) throw()
{
	if ( side == S_Side::Our_Side )
	{
		set_teammate_info( player_number ,
				   region , body_angle , face_angle );
	}
	else
	{
		set_opponent_info( player_number ,
				   region , body_angle , face_angle );
	}
}


#include  "d2_point.h"
void   Field_State::set_teammate_info( int  player_number ,
				       const D2_Region &  region ) throw()
{
	assert( 1 <= player_number
		 &&  player_number
			<= static_cast<int>(teammate_position.size()) );

	teammate_position[player_number - 1].region    = region;
	teammate_position[player_number - 1].have_info = true;
}

void   Field_State::set_teammate_info( int  player_number ,
				       const D2_Region &  region ,
				       double  body_angle )
					      throw()
{
	assert( 1 <= player_number
		 &&  player_number
			<= static_cast<int>(teammate_position.size()) );

	teammate_position[player_number - 1].region          = region;
	teammate_position[player_number - 1].have_info       = true;
	teammate_position[player_number - 1].body_angle      = body_angle;
	teammate_position[player_number - 1].have_body_angle = true;
}

void   Field_State::set_teammate_info( int  player_number ,
				       const D2_Region &  region ,
				       double  body_angle ,
				       double  face_angle )
					      throw()
{
	assert( 1 <= player_number
		 &&  player_number
			<= static_cast<int>(teammate_position.size()) );

	teammate_position[player_number - 1].region          = region;
	teammate_position[player_number - 1].have_info       = true;
	teammate_position[player_number - 1].body_angle      = body_angle;
	teammate_position[player_number - 1].have_body_angle = true;
	teammate_position[player_number - 1].face_angle      = face_angle;
	teammate_position[player_number - 1].have_face_angle = true;
}

void   Field_State::set_opponent_info( int  player_number ,
				       const D2_Region &  region )
					      throw()
{
	assert( 1 <= player_number
		 &&  player_number
			<= static_cast<int>(opponent_position.size()) );

	opponent_position[player_number - 1].region    = region;
	opponent_position[player_number - 1].have_info = true;
}

void   Field_State::set_opponent_info( int  player_number ,
				       const D2_Region &  region ,
				       double  body_angle )
					      throw()
{
	assert( 1 <= player_number
		 &&  player_number
			<= static_cast<int>(opponent_position.size()) );

	opponent_position[player_number - 1].region          = region;
	opponent_position[player_number - 1].have_info       = true;
	opponent_position[player_number - 1].body_angle      = body_angle;
	opponent_position[player_number - 1].have_body_angle = true;
}

void   Field_State::set_opponent_info( int  player_number ,
				       const D2_Region &  region ,
				       double  body_angle ,
				       double  face_angle )
					      throw()
{
	assert( 1 <= player_number
		 &&  player_number
			<= static_cast<int>(opponent_position.size()) );

	opponent_position[player_number - 1].region          = region;
	opponent_position[player_number - 1].have_info       = true;
	opponent_position[player_number - 1].body_angle      = body_angle;
	opponent_position[player_number - 1].have_body_angle = true;
	opponent_position[player_number - 1].face_angle      = face_angle;
	opponent_position[player_number - 1].have_face_angle = true;
}

void   Field_State::set_my_info( const D2_Region &  region ) throw()
{
	this -> set_teammate_info( my_player_number , region );
}

void   Field_State::set_my_info( const D2_Region &  region ,
				 double  body_angle ) throw()
{
	this -> set_teammate_info( my_player_number , region , body_angle );
}

void   Field_State::set_my_info( const D2_Region &  region ,
				 double  body_angle ,
				 double  face_angle ) throw()
{
	this -> set_teammate_info( my_player_number ,
				   region , body_angle , face_angle );
}



//
// Get Information
//
const D2_Region &  Field_State::ball_region() const // throw( exception )
{
	if ( ball_position.have_info )
	{
		return( ball_position.region );
	}
	else
	{
		throw exception();
	}
}

const D2_Region &  Field_State::ball_vector() const // throw( exception )
{
	if ( ball_position.have_vector )
	{
		return( ball_position.vec );
	}
	else
	{
		throw exception();
	}
}

const D2_Region &  Field_State::player_region( S_Side  side ,
					       int  player_number )
	const // throw( exception )
{
	switch( side )
	{
	case S_Side::Our_Side:
		return( this -> teammate_region( player_number ) );
		break;

	case S_Side::Opponent_Side:
		return( this -> opponent_region( player_number ) );
		break;

	default:
		{
			throw exception();
		}
		break;
	}
}


double  Field_State::player_body_angle( S_Side  side , int  player_number )
	const // throw( exception )
{
	switch( side )
	{
	case S_Side::Our_Side:
		return( this -> teammate_body_angle( player_number ) );
		break;

	case S_Side::Opponent_Side:
		return( this -> opponent_body_angle( player_number ) );
		break;

	default:
		{
			throw exception();
		}
	}
}

double  Field_State::player_face_angle( S_Side  side , int  player_number )
	const // throw( exception )
{
	switch( side )
	{
	case S_Side::Our_Side:
		return( this -> teammate_face_angle( player_number ) );
		break;
	case S_Side::Opponent_Side:
		return( this -> opponent_face_angle( player_number ) );
		break;
	default:
		{
			throw exception();
		}
	}
}

const D2_Region &  Field_State::my_region() const // throw( exception )
{
	return( this -> teammate_region( my_player_number ) );
}

double  Field_State::my_body_angle() const // throw( exception )
{
	return( this -> teammate_body_angle( my_player_number ) );
}

double  Field_State::my_face_angle() const // throw( exception )
{
	return( this -> teammate_face_angle( my_player_number ) );
}

const D2_Region &  Field_State::teammate_region( int  player_number )
	const // throw( exception )
{
	if ( ! ( 1 <= player_number
		  &&  player_number
			  <= static_cast<int>(teammate_position.size()) )
	  || ! teammate_position[player_number - 1].have_info )
	{
		throw exception();
	}

	return( teammate_position[player_number - 1].region );
}


double  Field_State::teammate_body_angle( int  player_number )
	const // throw( exception )
{
	if ( ! ( 1 <= player_number
		  &&  player_number
			  <= static_cast<int>(teammate_position.size()) )
	  || ! teammate_position[player_number - 1].have_info
	  || ! teammate_position[player_number - 1].have_body_angle )
	{
		throw exception();
	}

	return( teammate_position[player_number - 1].body_angle );
}

double  Field_State::teammate_face_angle( int  player_number )
	const //throw( exception )
{
	if ( ! ( 1 <= player_number
		  &&  player_number
			  <= static_cast<int>(teammate_position.size()) )
	  || ! teammate_position[player_number - 1].have_info
	  || ! teammate_position[player_number - 1].have_face_angle )
	{
		throw exception();
	}

	return( teammate_position[player_number - 1].face_angle );
}

const D2_Region &  Field_State::opponent_region( int  player_number )
	const // throw( exception )
{
	if ( ! ( 1 <= player_number
		  &&  player_number
			  <= static_cast<int>(opponent_position.size()) )
	  || ! opponent_position[player_number - 1].have_info )
	{
		throw exception();
	}

	return( opponent_position[player_number - 1].region );
}


double  Field_State::opponent_body_angle( int  player_number )
	const // throw( exception )
{
	if ( ! ( 1 <= player_number
		  &&  player_number
			  <= static_cast<int>(opponent_position.size()) )
	  || ! opponent_position[player_number - 1].have_info
	  || ! opponent_position[player_number - 1].have_body_angle )
	{
		throw exception();
	}

	return( opponent_position[player_number - 1].body_angle );
}


double  Field_State::opponent_face_angle( int  player_number )
	const // throw( exception )
{
	if ( ! ( 1 <= player_number
		  &&  player_number
			  <= static_cast<int>(opponent_position.size()) )
	  || ! opponent_position[player_number - 1].have_info
	  || ! opponent_position[player_number - 1].have_face_angle )
	{
		throw exception();
	}

	return( opponent_position[player_number - 1].face_angle );
}
