#ifndef	   FIELD_STATE_H_INCLUDED
#define	   FIELD_STATE_H_INCLUDED

#include  "d2_region.h"
#include  "s_basic.h"
#include  <vector>
#include  <stdexcept>
#include  <string>

class  Field_State
{
protected:
	int	my_player_number;

	struct  Player_Position
	{
		bool		have_info;
		D2_Region	region;

		bool		have_body_angle;
		double		body_angle;

		bool		have_face_angle;
		double		face_angle;
#if 0
		bool		have_vector;
		D2_Region	vec;
#endif
		Player_Position()
			: have_info( false ) , region() ,
			  have_body_angle( false ) , body_angle( 0.0 ) ,
			  have_face_angle( false ) , face_angle( 0.0 )
			{
			}
	};

	struct  Ball_Position
	{
		bool		have_info;
		D2_Region	region;

		bool		have_vector;
		D2_Region	vec;

		Ball_Position()
			: have_info( false ) , region() ,
			  have_vector( false ) , vec()
			{
			}
	};

	std::vector<Player_Position>	teammate_position;
	std::vector<Player_Position>	opponent_position;
#if 0
	std::vector<Player_Position>	unknown_player_position;
#endif
	Ball_Position			ball_position;

public:
	 Field_State( int  my_player_num ,
		      size_t  n_teammate = ELEVEN ,
		      size_t  n_opponent = ELEVEN ) throw();

	 Field_State( const std::string &  str );
	~Field_State();

	//
	// Set Information
	//
	void		set_ball_info( const D2_Region &  region ) throw();

	void		set_ball_info( const D2_Region &  region ,
				       const D2_Region &  vec ) throw();

	void		set_player_info( S_Side  side ,  int  player_number ,
					 const D2_Region &  region ) throw();

	void		set_player_info( S_Side  side ,  int  player_number ,
					 const D2_Region &  region ,
					 double  body_angle ) throw();

	void		set_player_info( S_Side  side ,  int  player_number ,
					 const D2_Region &  region ,
					 double  body_angle ,
					 double  face_angle ) throw();

	void		set_teammate_info( int  player_number ,
					   const D2_Region &  region ) throw();

	void		set_teammate_info( int  player_number ,
					   const D2_Region &  region ,
					   double  body_angle) throw();

	void		set_teammate_info( int  player_number ,
					   const D2_Region &  region ,
					   double  body_angle ,
					   double  face_angle ) throw();

	void		set_opponent_info( int  player_number ,
					   const D2_Region &  region ) throw();

	void		set_opponent_info( int  player_number ,
					   const D2_Region &  region,
					   double  body_angle ) throw();

	void		set_opponent_info( int  player_number ,
					   const D2_Region &  region ,
					   double  body_angle ,
					   double  face_angle ) throw();

	void		set_my_info( const D2_Region &  region ) throw();

	void		set_my_info( const D2_Region &  region ,
				     double  body_angle ) throw();

	void		set_my_info( const D2_Region &  region ,
				     double  body_angle ,
				     double  face_angle ) throw();


	//
	// Get Information
	//
	const D2_Region &	ball_region() const /*throw(std::exception)*/;
	const D2_Region &	ball_vector() const /*throw(std::exception)*/;

	const D2_Region &	player_region( S_Side  side ,
					       int  player_number )
					const /*throw(std::exception)*/;

	double			player_body_angle( S_Side  side ,
						   int  player_number )
					const /*throw(std::exception)*/;

	double			player_face_angle( S_Side  side ,
						   int  player_number )
					const /*throw(std::exception)*/;

	const D2_Region &	my_region()
					const /*throw(std::exception)*/;

	double			my_body_angle()
					const /*throw(std::exception)*/;

	double			my_face_angle()
					const /*throw(std::exception)*/;

	const D2_Region &	teammate_region( int  player_number )
					const /*throw(std::exception)*/;

	double			teammate_body_angle( int  player_number )
					const /*throw(std::exception)*/;

	double			teammate_face_angle( int  player_number )
					const /*throw(std::exception)*/;

	const D2_Region &	opponent_region( int  player_number )
					const /*throw(std::exception)*/;

	double			opponent_body_angle( int  player_number )
					const /*throw(std::exception)*/;

	double			opponent_face_angle( int  player_number )
					const /*throw(std::exception)*/;
};


#endif	/* FIELD_STATE_H_INCLUDED */
