#include  "sight_info_analyzer.h"
#include  "s_basic.h"
#include  "dig_rad.h"
#include  <cmath>
#include  <algorithm>
#include  <stdexcept>

using namespace std;

Sight_Info_Analyzer::Sight_Info_Analyzer(
				 const Sight_Info_from_Server &  sight ,
				 double  relative_face_angle )
	: static_obj() , line() ,
	  sorted_by_short_distance( false ) , obj_sorted_by_short_distance() ,
	  have_my_coordinate( false ) ,
	  my_coordinate( D2_Vector::XY , 0.0 , 0.0 ) ,
	  have_my_angle( false ) , my_body_angle( 0.0 ) ,
	  my_relative_face_angle( relative_face_angle ) ,
	  my_absolute_face_angle( my_body_angle + relative_face_angle ) ,
	  have_ball_player_coordinate( false ) , ball_info() ,
	  teammate_info(ELEVEN) , opponent_info(ELEVEN)
{
	set_flag_goal_info( sight );
	set_line_info( sight );
	set_ball_info( sight );
	set_player_info( sight );
}

void   Sight_Info_Analyzer::set_flag_goal_info(
				 const Sight_Info_from_Server &  sight )
{
	// flag
	for( size_t  i = 0  ;  i < sight.flag.size()  ;  i ++ )
	{
		if ( sight.flag[i].relative.has_direction
		  && sight.flag[i].relative.has_distance
		  && sight.flag[i].relative.in_sight )
		{
			Object_Coordinate_Info	info;

			info.absolute = sight.flag[i].entity.location;
			info.relative = sight.flag[i].relative;

			static_obj.push_back( info );
		}
		else
		{
			// XXX
		}
	}

	//   goal
	for( size_t  i = 0  ;  i < sight.goal.size()  ;  i ++ )
	{
		if ( sight.goal[i].relative.has_direction
		  && sight.goal[i].relative.has_distance
		  && sight.goal[i].relative.in_sight )
		{
			Object_Coordinate_Info	info;

			info.absolute = sight.goal[i].entity.location;
			info.relative = sight.goal[i].relative;

			static_obj.push_back( info );
		}
		else
		{
			// XXX
		}
	}
}

void   Sight_Info_Analyzer::set_line_info(
				 const Sight_Info_from_Server &  sight )
{
	//   line
	line = sight.line;
}

void   Sight_Info_Analyzer::set_ball_info(
				 const Sight_Info_from_Server &  sight )
{
	ball_info.absolute = D2_Vector( D2_Vector::XY , 0.0 , 0.0 );
	ball_info.relative = D2_Vector( D2_Vector::XY , 0.0 , 0.0 );
	ball_info.recognized = false;
	ball_info.have_vector = false;
	ball_info.relative_vector = D2_Vector( D2_Vector::XY , 0.0 , 0.0 );

	if ( sight.ball.size() == 1 )
	{
		ball_info.recognized = true;
		ball_info.relative = sight.ball[0].relative;

		if ( sight.ball[0].relative.has_d_distance
		 &&  sight.ball[0].relative.has_d_direction )
		{
			ball_info.have_vector = true;
			D2_Vector	to;

			to.set( D2_Vector::Pole ,
				ball_info.relative.r()
					+ sight.ball[0].relative.d_distance ,
				ball_info.relative.theta()
					+ sight.ball[0].relative.d_direction );
			ball_info.relative_vector = to - ball_info.relative;
		}
	}
}

void   Sight_Info_Analyzer::set_player_info(
				 const Sight_Info_from_Server &  sight )
{
	// initialize
	for ( size_t  i = 0  ;  i < teammate_info.size()  ;  i ++ )
	{
		teammate_info[i].recognized = false;

		teammate_info[i].absolute = D2_Vector( D2_Vector::XY ,
						       0.0 , 0.0 );

		teammate_info[i].relative = D2_Vector( D2_Vector::XY ,
						       0.0 , 0.0 );

		teammate_info[i].have_vector = false;

		teammate_info[i].relative_vector
			= D2_Vector( D2_Vector::XY , 0.0 , 0.0 );
	}

	for ( size_t  i = 0  ;  i < opponent_info.size()  ;  i ++ )
	{
		opponent_info[i].recognized = false;

		opponent_info[i].absolute = D2_Vector( D2_Vector::XY ,
							0.0 , 0.0 );
		opponent_info[i].relative = D2_Vector( D2_Vector::XY ,
							0.0 , 0.0 );
		opponent_info[i].have_vector = false;

		opponent_info[i].relative_vector
			= D2_Vector( D2_Vector::XY , 0.0 , 0.0 );
	}

	// overwrite
	for ( size_t  i = 0  ;  i < sight.player.size()  ;  i ++ )
	{
		const Player_Info_from_Server &  pl = sight.player[i];

		if ( pl.entity.player_number <= 0
		  || pl.entity.side == S_Side::Unknown )
		{
			continue;
		}

		if ( pl.entity.side == S_Side::Our_Side )
		{
			teammate_info[pl.entity.player_number - 1].relative
				= pl.relative;

			teammate_info[pl.entity.player_number - 1].recognized
				= true;

			if ( pl.relative.has_d_distance
			 &&  pl.relative.has_d_direction )
			{
				D2_Vector	to;

				to.set( D2_Vector::Pole ,
					teammate_info[
					  pl.entity.player_number - 1]
					    .relative.r()
					+ pl.relative.d_distance ,
					D2_Vector(pl.relative).theta()
					+ pl.relative.d_direction );

				teammate_info[pl.entity.player_number - 1]
					.relative_vector
				  = to - teammate_info[
					 pl.entity.player_number - 1].relative;

				teammate_info[pl.entity.player_number - 1]
					.have_vector = true;

			}
		}
		else
		{
			opponent_info[pl.entity.player_number - 1].relative
				= pl.relative;

			opponent_info[pl.entity.player_number - 1].recognized
				= true;

			if ( pl.relative.has_d_distance
			 &&  pl.relative.has_d_direction )
			{
				D2_Vector	to;

				to.set( D2_Vector::Pole ,
					opponent_info[
					  pl.entity.player_number - 1]
					    .relative.r()
					+ pl.relative.d_distance ,
					D2_Vector(pl.relative).theta()
					+ pl.relative.d_direction );

				opponent_info[pl.entity.player_number - 1]
					.relative_vector
				  = to - opponent_info[
					 pl.entity.player_number - 1].relative;

				opponent_info[pl.entity.player_number - 1]
					.have_vector = true;
			}
		}
	}
}




Sight_Info_Analyzer::~Sight_Info_Analyzer()
{
}


#include  "d2_point.h"
Field_State  Sight_Info_Analyzer::get_field_state( int  my_player_number )
	// throw( exception )
{
	try
	{
		if ( ! (1 <= my_player_number
			&& my_player_number
			     <= static_cast<int>(teammate_info.size()) ) )
		{
			throw exception();
		}

		require_ball_player_coordinate();

		Field_State	s( my_player_number ,
				   teammate_info.size() ,
				   opponent_info.size() );

		if ( have_my_coordinate && have_my_angle )
		{
			s.set_my_info( ref_count_ptr<const D2_Region_Entity>(
					   new D2_Point( my_coordinate ) ) ,
				       my_body_angle ,
				       my_relative_face_angle );
		}
		else if ( have_my_coordinate )
		{
			s.set_my_info( ref_count_ptr<const D2_Region_Entity>(
				 new D2_Point( my_coordinate ) ) );
		}
		else
		{
			throw exception();
		}

		if ( ball_info.recognized )
		{
			if ( ball_info.have_vector )
			{
				s.set_ball_info(
				  ref_count_ptr<const D2_Region_Entity>(
				   new D2_Point( ball_info.absolute ) ) ,
				  ref_count_ptr<const D2_Region_Entity>(
				   new D2_Point( ball_info.absolute_vector )));
			}
			else
			{
				s.set_ball_info(
				    ref_count_ptr<const D2_Region_Entity>(
				      new D2_Point( ball_info.absolute ) ) );
			}
		}

		for ( size_t  i = 0  ;  i < teammate_info.size()  ;  i ++ )
		{
			if ( teammate_info[i].recognized )
			{
				// XXX: set absolute vector
				s.set_teammate_info( i + 1 ,
				     ref_count_ptr<const D2_Region_Entity>(
				 new D2_Point( teammate_info[i].absolute ) ) );
			}
		}

		for ( size_t  i = 0  ;  i < opponent_info.size()  ;  i ++ )
		{
			if ( opponent_info[i].recognized )
			{
				// XXX: set absolute vector
				s.set_opponent_info( i + 1 ,
				    ref_count_ptr<const D2_Region_Entity>(
				 new D2_Point( opponent_info[i].absolute ) ) );
			}
		}

		return( s );
	}
	catch( ... )
	{
		throw exception();
	}
}


size_t  Sight_Info_Analyzer::n_objects() throw()
{
	return( static_obj.size() );
}


size_t  Sight_Info_Analyzer::n_lines() throw()
{
	return( line.size() );
}


void  Sight_Info_Analyzer::require_obj_sorted_by_short_distance()
{
	if ( ! sorted_by_short_distance )
	{
		class  Compare_by_Short_Distance
		{
		public:
		static	bool	compare( const Object_Coordinate_Info &  a ,
					 const Object_Coordinate_Info &  b )
			{
				return( a.relative.r() < b.relative.r() );
			}
		};

		obj_sorted_by_short_distance = static_obj;
		sort( obj_sorted_by_short_distance.begin() ,
		      obj_sorted_by_short_distance.end() ,
		      Compare_by_Short_Distance::compare );

		sorted_by_short_distance = true;
	}
}

void   Sight_Info_Analyzer::require_my_coordinate()
	// throw( exception )
{
	try
	{
		get_my_coordinate( &my_coordinate , &my_absolute_face_angle );

		my_body_angle
			= my_absolute_face_angle - my_relative_face_angle;

		have_my_angle      = true;
		have_my_coordinate = true;
	}
	catch( ... )
	{
		throw exception();
	}
}


void   Sight_Info_Analyzer::require_ball_player_coordinate()
	// throw( exception )
{
	try
	{
		require_my_coordinate();
	}
	catch( ... )
	{
		throw exception();
	}

	if ( ball_info.recognized )
	{
		D2_Vector	b = ball_info.relative;
		ball_info.absolute
			= my_coordinate + b.rotate( my_absolute_face_angle );

		if ( ball_info.have_vector )
		{
			ball_info.absolute_vector
			    = ball_info.relative_vector
					  .rotate( my_absolute_face_angle );
		}
	}

	for ( size_t  i = 0  ;  i < teammate_info.size()  ;  i ++ )
	{
		if ( teammate_info[i].recognized )
		{
			D2_Vector	p = teammate_info[i].relative;

			teammate_info[i].absolute
			  = my_coordinate + p.rotate( my_absolute_face_angle );

			if ( teammate_info[i].have_vector )
			{
				teammate_info[i].absolute_vector
				  = teammate_info[i].relative_vector
					  .rotate( my_absolute_face_angle );
			}
		}
	}

	for ( size_t  i = 0  ;  i < opponent_info.size()  ;  i ++ )
	{
		if ( opponent_info[i].recognized )
		{
			D2_Vector	p = opponent_info[i].relative;

			opponent_info[i].absolute
			  = my_coordinate + p.rotate( my_absolute_face_angle );

			if ( opponent_info[i].have_vector )
			{
				opponent_info[i].absolute_vector
				  = opponent_info[i].relative_vector
					  .rotate( my_absolute_face_angle );
			}
		}
	}

	have_ball_player_coordinate = true;
}




#include  "field_recog.h"
#include  "quantize.h"
#if 1
#include  <fstream>
#include  "field_recog.h"
#endif

#if 0
# if 1
# include  <cstdio>
# endif
#include  "d2_composite_straight_line_divided_region_interface.h"

void   Sight_Info_Analyzer::get_my_coordinate( D2_Vector *  res_vec ,
					       double *  res_ang )
	// throw( exception )
{
	*res_ang = 0.0;
	(*res_vec).set( D2_Vector::XY , 0.0 , 0.0 );

	double	angle = get_my_angle_from_line();

	if ( angle > dig_to_rad( 0.5 ) )
	{
		angle += dig_to_rad( 0.5 );
	}
	else if ( angle < - dig_to_rad( 0.5 ) )
	{
		angle -= dig_to_rad( 0.5 );
	}

	*res_ang = angle;

	require_obj_sorted_by_short_distance();

	vector<Object_Coordinate_Info> &	objects
				= obj_sorted_by_short_distance;

	D2_Composite_Straight_Line_Divided_Region_Interface	region;

	for ( size_t  i = 0  ;  i < objects.size()  ;  i ++ )
	{
		D2_Vector	cood = objects[i].absolute;
		D2_Vector	vec  = objects[i].relative;

		if ( vec.theta() > dig_to_rad( 0.5 ) )
		{
			vec.set( D2_Vector::Pole ,
				  vec.r() ,
				  vec.theta() + dig_to_rad( 0.5 ) );
		}
		else if ( vec.theta() < - dig_to_rad( 0.5 ) )
		{
			vec.set( D2_Vector::Pole ,
				  vec.r() ,
				  vec.theta() - dig_to_rad( 0.5 ) );
		}

		double		point_abs_angle = vec.theta() + angle;

		if ( vec.r() <= 0.1 )
		{
			(*res_vec) = cood;
			return;
		}

#if 1
		region.add( cood , point_abs_angle + dig_to_rad(1.0 + 0.01) ,
			    cood + D2_Vector( D2_Vector::Pole , 1.0 ,
					      point_abs_angle
					        + dig_to_rad( 90.0 ) ) );

		region.add( cood , point_abs_angle - dig_to_rad(1.0 + 0.01) ,
			    cood + D2_Vector( D2_Vector::Pole , 1.0 ,
					      point_abs_angle
					        - dig_to_rad( 90.0 ) ) );
#endif

#if 1
		double	r_min;
		double	r_max;
		far_object_unquantized_distance( vec.r() , &r_min , &r_max ,
						 0.1 );

		r_min -= 0.01;
		r_max += 0.01;

		D2_Vector	max_point_vector( D2_Vector::Pole ,
						  r_max ,
						  point_abs_angle
						    + dig_to_rad( 180.0 ) );

		D2_Vector	max_point = cood + max_point_vector;

		region.add( max_point , point_abs_angle + dig_to_rad( 90.0 ) ,
			    cood );

		D2_Vector	min_point_vector_1( D2_Vector::Pole ,
						    r_min ,
						    point_abs_angle
						     + dig_to_rad( 180.0 )
						     + dig_to_rad(   1.0
								    + 0.01 ) );

		D2_Vector	min_point_vector_2( D2_Vector::Pole ,
						    r_min ,
						    point_abs_angle
						      + dig_to_rad( 180.0 )
						      - dig_to_rad(   1.0
								    + 0.01 ) );

		D2_Vector	min_point_1 = cood + min_point_vector_1;
		D2_Vector	min_point_2 = cood + min_point_vector_2;

		region.add( min_point_vector_1 , min_point_vector_2 ,
			    max_point );
#endif
	}

	try
	{
		(*res_vec) = region.barycenter();
		D2_Vector	true_my_coordinate
			= Field_Recog::server_debug_info().true_my_coordinate;

#if 0
		fprintf( stderr , "% 3.4f" , region.area() );
#endif

		if ( ! region.in_region( true_my_coordinate ) )
		{
			cerr << "!!! ";
		}

	} catch(...){}
}
#else
void   Sight_Info_Analyzer::get_my_coordinate( D2_Vector *  res_vec ,
					       double *  res_ang )
	// throw( exception )
{
	*res_ang = 0.0;
	(*res_vec).set( D2_Vector::XY , 0.0 , 0.0 );

	double	angle = normalize_angle( get_my_angle_from_line() );

	if ( angle > dig_to_rad( 0.5 ) )
	{
		angle += dig_to_rad( 0.5 );
	}
	else if ( angle < - dig_to_rad( 0.5 ) )
	{
		angle -= dig_to_rad( 0.5 );
	}

	*res_ang = angle;

#if 0
	D2_Vector	cood;
	D2_Vector	dist;

	require_obj_sorted_by_short_distance();

	if ( obj_sorted_by_short_distance.size() == 0 )
	{
		throw exception();
	}

	cood = obj_sorted_by_short_distance[0].absolute;
	dist = obj_sorted_by_short_distance[0].relative;

	double	dist_angle = normalize_angle( dist.theta() );

	if ( dist_angle > dig_to_rad( 0.5 ) )
	{
		dist_angle += dig_to_rad( 0.5 );
	}
	else if ( dist_angle < - dig_to_rad( 0.5 ) )
	{
		dist_angle -= dig_to_rad( 0.5 );
	}

	D2_Vector	v( D2_Vector::Pole ,
			   dist.r() , dist_angle + angle + M_PI );
	(*res_vec) = cood + v;
#endif
#if 0
	pair<Object_Coordinate_Info,Object_Coordinate_Info>	two_objs
		= get_differ_angle_two_objs();

	(*res_vec) = get_my_coordinate_by_angle_and_two_point(
				angle , two_objs.first , two_objs.second );
#endif
#if 0
	require_obj_sorted_by_short_distance();

	D2_Vector	vec( D2_Vector::XY , 0.0 , 0.0 );
	size_t		n = 0;

	for ( size_t  i = 0  ;
	      i < obj_sorted_by_short_distance.size()  ;  i ++ )
	{
		for ( size_t  j = i + 1  ;
		      j < obj_sorted_by_short_distance.size()  ;  j ++ )
		{
			if( abs(
			   obj_sorted_by_short_distance[i].relative.theta()
			   - obj_sorted_by_short_distance[j].relative.theta() )
			    < dig_to_rad(15.0) )
			{
				continue;
			}

			vec += get_my_coordinate_by_angle_and_two_point(
				     angle ,
				     obj_sorted_by_short_distance[i] ,
				     obj_sorted_by_short_distance[j] );
			n ++;
		}
	}

	if ( n == 0 )
	{
		throw exception();
	}

	(*res_vec) = vec / n;
#endif
#if 0
	require_obj_sorted_by_short_distance();

	if ( obj_sorted_by_short_distance.size() == 0 )
	{
		throw exception();
	}


	D2_Vector	vec( D2_Vector::XY , 0.0 , 0.0 );
	double		n = 0.0;
	double		c = 1.0;

	for ( size_t  i = 0  ;
	      i < obj_sorted_by_short_distance.size()  ;  i ++ )
	{
		D2_Vector	cood;
		D2_Vector	dist;

		cood = obj_sorted_by_short_distance[0].absolute;
		dist = obj_sorted_by_short_distance[0].relative;

		double	dist_angle = normalize_angle( dist.theta() );

		if ( dist_angle > dig_to_rad( 0.5 ) )
		{
			dist_angle += dig_to_rad( 0.5 );
		}
		else if ( dist_angle < - dig_to_rad( 0.5 ) )
		{
			dist_angle -= dig_to_rad( 0.5 );
		}

		D2_Vector	v( D2_Vector::Pole , dist.r() ,
				   dist_angle + angle + M_PI );

		vec += (c * (cood + v));

		n += c;
		c *= 0.7;
	}

	(*res_vec) = vec / n;
#endif
#if 1
	require_obj_sorted_by_short_distance();

	if ( obj_sorted_by_short_distance.size() == 0 )
	{
		throw exception();
	}

#if 0
	static	fstream	log( "coordinate-log" , ios::app );

	if ( log )
	{
		const Debug_Info_from_Server &	debug
			= Field_Recog::server_debug_info();

		log << Field_Recog::server_debug_info_time() << " "
		    << debug.true_my_coordinate.x() << " "
		    << debug.true_my_coordinate.y() << " "
		    << debug.true_my_body_angle << " "
		    << debug.true_my_face_angle << "\t";

		for ( size_t  i = 0  ;
		      i < obj_sorted_by_short_distance.size()  ;  i ++ )
		{
			log << " "
			    << obj_sorted_by_short_distance[i].relative.x()
			    << " "
			    << obj_sorted_by_short_distance[i].relative.y()
			    << " "
			    << obj_sorted_by_short_distance[i].absolute.x()
			    << " "
			    << obj_sorted_by_short_distance[i].absolute.y();
		}

		log << endl;
	}
#endif

	D2_Vector	vec( D2_Vector::XY , 0.0 , 0.0 );
	double		n = 0.0;
	double		c = 1.0;

	for ( size_t  i = 0  ;
	      i < obj_sorted_by_short_distance.size()  ;  i ++ )
	{
		D2_Vector	cood;
		D2_Vector	dist;

#if 0
		cood = obj_sorted_by_short_distance[i].absolute;
		dist = obj_sorted_by_short_distance[i].relative;
#else
		cood = obj_sorted_by_short_distance[0].absolute;
		dist = obj_sorted_by_short_distance[0].relative;
#endif

		double	dist_angle = normalize_angle( dist.theta() );

		if ( dist_angle > dig_to_rad( 0.5 ) )
		{
			dist_angle += dig_to_rad( 0.5 );
		}
		else if ( dist_angle < - dig_to_rad( 0.5 ) )
		{
			dist_angle -= dig_to_rad( 0.5 );
		}

		D2_Vector	v( D2_Vector::Pole , dist.r() ,
				   dist_angle + angle + M_PI );

		double	e = exp( - 0.01 * dist.r() );

		vec += e * c * (cood + v);

		n += e * c;

		c *= 0.8;
	}

	(*res_vec) = vec / n;
#endif
}
#endif




double  Sight_Info_Analyzer::get_my_angle_from_line() // throw( exception )
{
	// return my angle [- PI , PI)

	if ( line.size() == 0 )
	{
		throw exception();
	}

	D2_Vector	vec = line[0].relative;
	double		angle;

	if ( vec.theta() < 0 )
	{
		angle = - M_PI / 2.0 - vec.theta();
	}
	else
	{
		angle =   M_PI / 2.0 - vec.theta();
	}

	switch( line[0].entity.which_line )
	{
	case SObject_Line_Identifier::Opponent_Goal_Line:
		break;

	case SObject_Line_Identifier::Left_Wing_Line:
		angle += M_PI / 2.0;
		break;

	case SObject_Line_Identifier::Our_Goal_Line:
		angle += M_PI;
		break;

	case SObject_Line_Identifier::Right_Wing_Line:
		angle += 3.0 / 2.0 * M_PI;
		break;
	}

	if ( line.size() >= 2 )
	{
		angle += M_PI;
	}

	while ( angle >= M_PI )
	{
		angle -= 2.0 * M_PI;
	}

	return( angle );
}


D2_Vector  Sight_Info_Analyzer::get_my_coordinate_by_angle_and_two_point(
			   double  angle ,
			   const Object_Coordinate_Info &  obj_1 ,
			   const Object_Coordinate_Info &  obj_2 )
{
	D2_Vector	u1 = obj_1.relative;
	D2_Vector	u2 = obj_2.relative;

	D2_Vector	p1 = obj_1.absolute;
	D2_Vector	p2 = obj_2.absolute;
	D2_Vector	p  = p1 - p2;

	u1 = u1.rotate( angle );
	u2 = u2.rotate( angle );

	u1 = u1.normalize();
	u2 = u2.normalize();

#if 0
	cout<<"u1 = "<<u1<<"("<<u1.r()<<","<<u1.theta()/M_PI*180.0<<")"<< endl;
	cout<<"u2 = "<<u2<<"("<<u2.r()<<","<<u2.theta()/M_PI*180.0<<")"<< endl;
	cout<<"p1 = "<<p1<<"("<<p1.r()<<","<<p1.theta()/M_PI*180.0<<")"<< endl;
	cout<<"p2 = "<<p2<<"("<<p2.r()<<","<<p2.theta()/M_PI*180.0<<")"<< endl;
	cout<<"p  = "<<p <<"("<<p .r()<<","<<p .theta()/M_PI*180.0<<")"<< endl;
#endif

	double	n = (u2.x() * p.y()  -  u2.y() * p.x())
		    / (u2.y() * u1.x()  -  u2.x() * u1.y() );

#if 0
	cout << "a = (u2.x() * p.y()  -  u2.y() * p.x()) = "
	     << (u2.x() * p.y()  -  u2.y() * p.x()) << endl;

	cout << "b = (u2.y() * u1.x()  -  u2.x() * u1.y() ) = "
	     << (u2.y() * u1.x()  -  u2.x() * u1.y() ) << endl;

	cout << "n  = " << n << endl;
#endif

	return( (p1  +  n * u1) );
}


pair<Sight_Info_Analyzer::Object_Coordinate_Info ,
     Sight_Info_Analyzer::Object_Coordinate_Info>
	Sight_Info_Analyzer::get_differ_angle_two_objs()
{
	const	double	eps = 15.0 / 180.0 * M_PI;

	require_obj_sorted_by_short_distance();

	for( size_t  i = 0  ;  i < static_obj.size()  ;  i ++ )
	{
		for( size_t  j = i + 1  ;  j < static_obj.size()  ;  j ++ )
		{
			if ( abs(   static_obj[i].relative.theta()
				  - static_obj[j].relative.theta() ) > eps )
			{
				pair<Sight_Info_Analyzer::
					Object_Coordinate_Info ,
				     Sight_Info_Analyzer::
					Object_Coordinate_Info>  ret;
				ret.first  = static_obj[i];
				ret.second = static_obj[j];
				return( ret );
			}
		}
	}

	throw exception();
}


void   Sight_Info_Analyzer::get_ball_coordinate( D2_Vector *  vec )
	// throw( exception )
{
	if ( ! ball_info.recognized )
	{
		throw exception();
	}

	try
	{
		require_ball_player_coordinate();
	}
	catch( ... )
	{
		throw exception();
	}

	*vec =  ball_info.absolute;
}


void   Sight_Info_Analyzer::get_player_coordinate( S_Side  side ,
						  int  player_number ,
						  D2_Vector *  vec )
	// throw( exception )
{
	switch( side )
	{
	case S_Side::Our_Side:
		{
			if ( ! (1 <= player_number
				  && player_number
				    <= static_cast<int>(teammate_info.size()) )
			  || ! teammate_info[player_number - 1].recognized )
			{
				throw exception();
			}

			*vec = teammate_info[player_number - 1].absolute;
		}
		break;

	case S_Side::Opponent_Side:
		{
			if ( ! (1 <= player_number
				  && player_number
				    <= static_cast<int>(opponent_info.size()) )
			  || ! opponent_info[player_number].recognized )
			{
				throw exception();
			}

			*vec = opponent_info[player_number].absolute;
		}
		break;

	case S_Side::Unknown:
		{
			throw exception();
		}
	}
}
