#include  "soccer_interface.h"
#include  "sserver_param.h"
#include  "dig_rad.h"
#include  <unistd.h>

using namespace std;

Soccer_Interface::Soccer_Interface( SServer_Connection_with_Game_Info &  serv ,
				    const SServer_Param &  p )
	: server_connection( serv ) , params( p ) ,
	  field_recog() ,
	  last_sight_info_turn( 0 ) , last_body_info_turn( 0 ) ,
	  last_sight_info_time_stamp() , last_body_info_time_stamp() ,
	  game_over_flag( false )
{
	this -> prev_main_command.no_command();

	this -> auto_change_view_type.type = Auto_Change_View_Type::Manual;

	this -> auto_change_view_narrow_high_view_mode = View_Angle::Normal;
}

Soccer_Interface::~Soccer_Interface()
{
}

void   Soccer_Interface::register_field_recog(
			      ref_count_ptr<Field_Recog_Abstract>  fr ) throw()
{
	field_recog.push_back( fr );
}



// protected
void   Soccer_Interface::register_info( const Information_from_Server &  info ,
					const Time_Stamp &  stamp ) throw()
{
	switch( info.type() )
	{
	case Information_from_Server::Type_Sight:
		if ( this -> last_sight_info_turn.turn != info.time() )
		{
			this -> last_sight_info_turn = info.time();
		}
		else if ( this -> last_sight_info_turn.sub_turn
			  <= this -> last_body_info_turn.sub_turn )
		{
			this -> last_sight_info_turn.sub_turn ++;
		}

		this -> last_sight_info_time_stamp = stamp;

		break;

	case Information_from_Server::Type_Body:
		if ( this -> last_body_info_turn.turn != info.time() )
		{
			this -> last_body_info_turn = info.time();
		}
		else if ( this -> last_body_info_turn.sub_turn
			  <= this -> last_sight_info_turn.sub_turn )
		{
			this -> last_body_info_turn.sub_turn ++;
		}

		this -> last_body_info_time_stamp = stamp;

		break;

	case Information_from_Server::Type_Whistle:
	    {
		const Whistle_Info_from_Server &	whistle
			= info.get_whistle_info();

		Judgement_Type	type = whistle.type;

		if ( type == Judgement_Type::Time_Up
		  || type == Judgement_Type::Time_Up_Without_A_Team
		  || (type == Judgement_Type::One_of_Play_Modes
		      && whistle.play_mode == Play_Mode::Time_Over) )
		{
			game_over_flag = true;
		}
	    }
		break;

	case Information_from_Server::Type_Error:
		cerr << "error message from server: ["
		     << info.get_error_info().error_message
		     << "]" << endl;
		return;
		break;

	default:
		break;
	}

	if ( info.illegal() )
	{
		cerr << "Unknown message receved!!" << endl;
		return;
	}

	for ( size_t  i = 0  ;  i < field_recog.size()  ;  i ++ )
	{
		field_recog[i] -> register_info( info );
	}
}


void   Soccer_Interface::wait_for_next_info(
			    Information_from_Server::Info_Type  t ) throw()
{
	Information_from_Server		info;
	bool				type_info_receved;
	bool				blocking;

	blocking = true;
	type_info_receved = false;
	for(;;)
	{
#if 1
		cout << "blocking = " << (blocking ? "true" : "false") << endl;
#endif

		Time_Stamp	stamp0;
		stamp0.set_current_time();

		info = server_connection.recv_info( blocking );

		Time_Stamp	stamp;
		stamp.set_current_time();

		if ( ! server_connection.respond() )
		{
			break;
		}

#if 1
		cout << "time = " << (stamp - stamp0).msec() << "msec" << endl;

		cout << info.raw_message();
#endif

		if ( info.type() == t )
		{
			type_info_receved = true;
			blocking = false;
		}
		else if ( info.no_info() )
		{
			if ( type_info_receved )
			{
				break;
			}
		}

		this -> register_info( info , stamp );
	}
}


void   Soccer_Interface::wait_for_next_turn( long  max_wait ) throw()
{
	this -> wait_for_next_info( Information_from_Server::Type_Body );

	Time_Stamp	stamp0;	// turn start time
	Time_Stamp	stamp;
	stamp0.set_current_time();
	stamp .set_current_time();

	while ( this -> last_sight_info_turn  !=  this -> last_body_info_turn )
	{
		// XXX: max_wait
		Information_from_Server	info
			= server_connection.recv_info( max_wait );

		stamp.set_current_time();

		if ( ! server_connection.respond() )
		{
			break;
		}

#if 1
		cout << "time = " << (stamp - stamp0).msec() << "msec" << endl;

		cout << "[" << info.raw_message() << "]" << endl;
#endif

		this -> register_info( info , stamp );
	}

#if 1
	//
	// Automatic Change_View
	//
	//   Warning: deprecated.
	//
	if ( this -> auto_change_view_type.type
	         == Auto_Change_View_Type::Narrow_High )
	{
		if ( this -> auto_change_view_narrow_high_view_mode
		     == View_Angle::Normal )
		{
			this -> send_change_view( View_Angle::Narrow ,
						  View_Quality::High );
			this -> auto_change_view_narrow_high_view_mode
				= View_Angle::Narrow;
		}
#if 0
		else if ( last_sight_info_time_stamp
			  < last_body_info_time_stamp.backward_msec( 20 ) )
		else if ( last_sight_info_time_stamp
			  < last_body_info_time_stamp.forward_msec( 20 ) )
		else if ( last_sight_info_time_stamp
			  < last_body_info_time_stamp )
#else
		else if ( last_sight_info_time_stamp
			  < last_body_info_time_stamp.backward_msec( 5 ) )
#endif
		{
			if ( this -> auto_change_view_narrow_high_view_mode
			     !=  View_Angle::Normal )
			{
				this -> send_change_view(View_Angle::Normal ,
							 View_Quality::High);
				this -> auto_change_view_narrow_high_view_mode
					= View_Angle::Normal;
			}
		}
	}
#endif

#ifdef TURN_NECK_OLD
	if ( field_recog.size() != 0 )
	{
		D2_Vector	my_coordinate( 0.0 , 0.0 );
		D2_Vector	ball_coordinate( 0.0 , 0.0 );
		D2_Vector	ball_vector( 0.0 , 0.0 );
		double		my_body_angle = 0.0;
		double		my_face_angle = 0.0;

		size_t		n_my_cood = 0;
		size_t		n_ball_cood = 0;
		size_t		n_ball_vector = 0;
		size_t		n_body_angle = 0;
		size_t		n_face_angle = 0;

		for ( size_t  i = 0  ;  i < field_recog.size()  ;  i ++ )
		{
			try
			{
				my_coordinate
				  += field_recog[i] -> get_my_coordinate();

				n_my_cood ++;
			} catch(...){}

			try
			{
				ball_coordinate
				  += field_recog[i] -> get_ball_coordinate();

				n_ball_cood ++;
			} catch(...){}

			try
			{
				ball_vector
					+= field_recog[i] -> get_ball_vector();
				n_ball_vector ++;
			} catch(...){}

			try
			{
				my_face_angle
				  += field_recog[i] -> get_my_body_angle();

				n_body_angle ++;
			} catch(...){}

			try
			{
				my_face_angle
				  += field_recog[i] -> get_my_face_angle();

				n_face_angle ++;
			} catch(...){}
		}

		if ( n_body_angle != 0
		  && n_face_angle != 0 )
		{
			my_body_angle /= n_body_angle;
			my_face_angle /= n_face_angle;

			double	moment;

			if ( n_my_cood   != 0
			  && n_ball_cood != 0 )
			{
				my_coordinate   /= n_my_cood;
				ball_coordinate /= n_ball_cood;

				if ( n_ball_vector != 0 )
				{
					ball_vector /= n_ball_vector;

					moment
					= normalize_angle(
					   (ball_coordinate + ball_vector
					    - my_coordinate).theta()
					   - (my_body_angle + my_face_angle) );
				}
				else
				{
					moment
					= normalize_angle(
					   (ball_coordinate
					    - my_coordinate).theta()
					   - (my_body_angle + my_face_angle) );
				}
			}
			else
			{
				moment = (- my_face_angle);
			}

			this -> send_turn_neck( moment );

			for ( size_t  i = 0  ;  i < field_recog.size()  ;
			      i ++ )
			{
				field_recog[i] -> turn_neck( moment );
			}
		}
	}
#endif

	for ( size_t  i = 0  ;  i < field_recog.size()  ;  i ++ )
	{
		// XXX: no check: command accepted or not
		field_recog[i] -> next_turn( prev_main_command , true );
	}
}


void   Soccer_Interface::wait_for_next_sight_info() throw()
{
	this -> wait_for_next_info( Information_from_Server::Type_Sight );
}




void   Soccer_Interface::set_automatic_change_view(
				       Auto_Change_View_Type::Type  t ) throw()
{
	this -> auto_change_view_type.type = t;
}




const SServer_Param &  Soccer_Interface::get_sserver_param() const throw()
{
	return( params );
}




bool   Soccer_Interface::game_is_over() const throw()
{
	return( game_over_flag );
}

bool   Soccer_Interface::server_no_response() const throw()
{
	return( ! server_connection.respond() );
}



int    Soccer_Interface::send_init( const string &  team_name ,  bool  goalie ,
				    Game_Info *  g_info ,
				    int  major_version ,  int  minor_version )
	throw()
{
	int	ret = server_connection.send_init( team_name ,
						   goalie ,
						   g_info ,
						   major_version ,
						   minor_version );

	if ( ! server_connection.registered() )
	{
		cerr << "not registered." << endl;
		return( -1 );
	}

	return( ret );
}

int    Soccer_Interface::send_reconnect( const string &  team_name ,
					 int  player_number ,
					 Game_Info *  g_info ,
					 long  interval_usec ) throw()
{
	int	ret = server_connection.send_reconnect( team_name ,
							player_number ,
							g_info ,
							interval_usec );

	if ( ! server_connection.registered() )
	{
		cerr << "not registered." << endl;
		return( -1 );
	}

	return( ret );
}

int    Soccer_Interface::send_bye( long  interval_usec ) throw()
{
	return( server_connection.send_bye( interval_usec ) );
}

int    Soccer_Interface::send_move( double  x ,  double  y ,
				    long  interval_usec ) throw()
{
	this -> prev_main_command.move( x , y );
	return( server_connection.send_move( x , y , interval_usec ) );
}

int    Soccer_Interface::send_dash( double  power ,
				    long  interval_usec ) throw()
{
	this -> prev_main_command.dash( power );
	return( server_connection.send_dash( power , interval_usec ) );
}

int    Soccer_Interface::send_kick( double  power ,  double  direction ,
				    long  interval_usec ) throw()
{
	this -> prev_main_command.kick( power , direction );
	return( server_connection.send_kick( power , direction ,
					     interval_usec ) );
}

int    Soccer_Interface::send_turn( double  moment ,
				    long  interval_usec ) throw()
{
	this -> prev_main_command.turn( moment );
	return( server_connection.send_turn( moment , interval_usec ) );
}

int    Soccer_Interface::send_say( const string &  message ,
				   long  interval_usec ) throw()
{
	return( server_connection.send_say( message , interval_usec ) );
}

int    Soccer_Interface::send_catch( double  direction ,
				     long  interval_usec ) throw()
{
	this -> prev_main_command.catch_command( direction );
	return( server_connection.send_catch( direction , interval_usec ) );
}

int    Soccer_Interface::send_change_view( View_Angle  angle ,
					   View_Quality  quality ,
					   long  interval_usec ) throw()
{
	return( server_connection.send_change_view( angle , quality ,
						    interval_usec ) );
}

int    Soccer_Interface::send_sense_body( long  interval_usec ) throw()
{
	return( server_connection.send_sense_body( interval_usec ) );
}

int    Soccer_Interface::send_turn_neck( double  moment ,
					 long  interval_usec ) throw()
{
	return( server_connection.send_turn_neck( moment ,
						  interval_usec ) );
}

int    Soccer_Interface::send_command( const Soccer_Command &  command ,
				       long  interval_usec ) throw()
{
	switch( command.type() )
	{
	case Soccer_Command::Say:
	case Soccer_Command::Change_View:
	case Soccer_Command::Sense_Body:
	case Soccer_Command::Turn_Neck:
		break;

	default:
		this -> prev_main_command = command;
	}

	return( server_connection.send_command( command , interval_usec ) );
}




int    Soccer_Interface::send_reconnect( const string &  team_name ,
					 int  player_number ,
					 bool  default_wait ) throw()
{
	return( server_connection.send_reconnect( team_name , player_number ,
						  NULL , default_wait ) );
}

int    Soccer_Interface::send_bye( bool  default_wait ) throw()
{
	return( server_connection.send_bye( default_wait ) );
}


int    Soccer_Interface::send_move( double  x ,  double  y ,
				    bool  default_wait ) throw()
{
	this -> prev_main_command.move( x , y );
	return( server_connection.send_move( x , y , default_wait ) );
}

int    Soccer_Interface::send_dash( double  power ,
				    bool  default_wait ) throw()
{
	this -> prev_main_command.dash( power );
	return( server_connection.send_dash( power , default_wait ) );
}

int    Soccer_Interface::send_kick( double  power ,  double  direction ,
				    bool  default_wait ) throw()
{
	this -> prev_main_command.kick( power , direction );
	return( server_connection.send_kick( power , direction ,
					     default_wait ) );
}

int    Soccer_Interface::send_turn( double  moment ,
				    bool  default_wait ) throw()
{
	this -> prev_main_command.turn( moment );
	return( server_connection.send_turn( moment , default_wait ) );
}

int    Soccer_Interface::send_say( const string &  message ,
				   bool  default_wait ) throw()
{
	this -> prev_main_command.say( message );
	return( server_connection.send_say( message , default_wait ) );
}

int    Soccer_Interface::send_catch( double  direction ,
				     bool  default_wait ) throw()
{
	this -> prev_main_command.catch_command( direction );
	return( server_connection.send_catch( direction , default_wait ) );
}

int    Soccer_Interface::send_change_view( View_Angle  angle ,
					   View_Quality  quality ,
					   bool  default_wait ) throw()
{
	this -> prev_main_command.change_view( angle , quality );
	return( server_connection.send_change_view( angle , quality ,
						    default_wait ) );
}

int    Soccer_Interface::send_sense_body( bool  default_wait ) throw()
{
	this -> prev_main_command.sense_body();
	return( server_connection.send_sense_body( default_wait ) );
}

int    Soccer_Interface::send_turn_neck( double  direction ,
					 bool  default_wait ) throw()
{
	this -> prev_main_command.turn_neck( direction );
	return( server_connection.send_turn_neck( direction , default_wait ) );
}

int    Soccer_Interface::send_command( const Soccer_Command &  command ,
				       bool  default_wait ) throw()
{
	switch( command.type() )
	{
	case Soccer_Command::Say:
	case Soccer_Command::Change_View:
	case Soccer_Command::Sense_Body:
	case Soccer_Command::Turn_Neck:
		break;

	default:
		this -> prev_main_command = command;
	}

#ifdef TURN_NECK_OLD
	// XXX
	Information_from_Server		info;

	while( ! (info = server_connection.recv_info( false )).no_info() )
	{
		Time_Stamp	stamp;
		stamp.set_current_time();

		if ( ! server_connection.respond() )
		{
			break;
		}

		this -> register_info( info , stamp );
	}
#endif

	return( server_connection.send_command( command , default_wait ) );
}


int    Soccer_Interface::send_composite_command
		( const Soccer_Composite_Command &  composite_command ,
		  bool  default_wait ) throw()
{
	return( this -> send_composite_command
		( composite_command ,
		  (default_wait ? server_connection.default_wait() : 0) ) );
}

int    Soccer_Interface::send_composite_command
		( const Soccer_Composite_Command &  composite_command ,
		  long  interval_usec ) throw()
{
	int	ret = -1;

	//
	// send main command
	//
	switch( composite_command.base_type() )
	{
	case Soccer_Command::Move:
	case Soccer_Command::Dash:
	case Soccer_Command::Kick:
	case Soccer_Command::Turn:
	case Soccer_Command::Catch:
	case Soccer_Command::Wait:
	case Soccer_Command::No_Command:
	case Soccer_Command::Illegal:
		ret = server_connection.send_command(
			   composite_command.base_command() , interval_usec );
		break;

	default:
		break;
	}

	if ( composite_command.base_type() == Soccer_Command::Illegal )
	{
		return( ret );
	}


	//
	// clear stream
	//
	Information_from_Server		info;
	while( ! (info = server_connection.recv_info( false )).no_info() )
	{
		Time_Stamp	stamp;
		stamp.set_current_time();

		if ( ! server_connection.respond() )
		{
			break;
		}

		this -> register_info( info , stamp );
	}


	//
	// send sub commands
	//
	if ( composite_command.turn_neck() )
	{
		if ( server_connection.send_command(
				    *(composite_command.turn_neck()) ,
				    interval_usec ) == -1 )
		{
			ret = -1;
		}

		for ( size_t  i = 0  ;  i < field_recog.size()  ;  i ++ )
		{
			field_recog[i]
			  -> turn_neck( composite_command.turn_neck()
					  -> turn_neck_direction() );
		}
	}


	if ( composite_command.change_view() )
	{
		if ( server_connection.send_command(
				    *(composite_command.change_view()) ,
				    interval_usec ) == -1 )
		{
			ret = -1;
		}
	}

	if ( composite_command.sense_body() )
	{
		if ( server_connection.send_command(
				    *(composite_command.sense_body()) ,
				    interval_usec ) == -1 )
		{
			ret = -1;
		}
	}

	if ( composite_command.say() )
	{
		if ( server_connection.send_command(
				    *(composite_command.say()) ,
				    interval_usec ) == -1 )
		{
			ret = -1;
		}
	}

	return( ret );
}
