#ifdef SSERVER_CONNECTION_TEST
#include  "sserver_connection.h"
#include  <iostream>
#include  <cstdlib>
#include  <cstddef>

extern "C"
{
#include  <unistd.h>
}

using namespace std;

int    main( void )
{
	SServer_Connection  server( "localhost" ,
				    SServer_Connection::SSERVER_DEFAULT_PORT );

	server.send_init( "test" );

	server.send_move( -3.0 , 0.12345678 );

	for(;;)
	{
		Information_from_Server	info;

		info = server.recv_info();
	}

	return( 0 );
}
#endif




#include  "sserver_connection.h"
#include  "sserver_message_translator.h"
#include  "sserver_io.h"

extern "C"
{
	#include  <sys/types.h>
	#include  <sys/socket.h>
	#include  <netinet/in.h>
	#include  <netdb.h>
	#include  <unistd.h>
#ifdef __osf__
	void	usleep( unsigned  useconds );
#endif
}
#include  <string>
using namespace std;

SServer_Connection::SServer_Connection( SServer_IO *  io ,
					bool  use_send_radian ,
					bool  use_recv_radian ,
					long  default_wait_time )
{
	this -> our_side = S_Side_LR::Unknown;
	this -> send_time.set_current_time();
	this -> use_send_radian = use_send_radian;
	this -> use_recv_radian = use_recv_radian;

	this -> default_wait_time = default_wait_time;

	this -> serv_io = io;
	this -> serv_io_must_delete = false;
}


SServer_Connection::SServer_Connection( const string &  host ,
					in_port_t  port ,
					bool  use_send_radian ,
					bool  use_recv_radian ,
					long  default_wait_time ,
					long  max_response )
{
	this -> our_side = S_Side_LR::Unknown;
	this -> send_time.set_current_time();
	this -> use_send_radian = use_send_radian;
	this -> use_recv_radian = use_recv_radian;

	this -> default_wait_time = default_wait_time;

	this -> serv_io = new SServer_Socket( host , port , max_response );
	this -> serv_io_must_delete = true;
}


SServer_Connection::~SServer_Connection()
{
	if ( serv_io_must_delete )
	{
		delete this -> serv_io;
		this -> serv_io = NULL;
	}
}


SServer_Connection::operator bool()
{
	return( serv_io != NULL  &&  serv_io -> operator bool() );
}

bool   SServer_Connection::respond()
{
	return( serv_io != NULL  &&  serv_io -> respond() );
}



long  SServer_Connection::default_wait() const
{
	return( default_wait_time );
}


int    SServer_Connection::send( const string &  str ,  long  interval_usec )
{
	if ( interval_usec != 0 )
	{
		Time_Stamp	current_time;
		current_time.set_current_time();

		long	diff;
		diff = (current_time - send_time).usec();

		if ( diff  <  interval_usec )
		{
			usleep( interval_usec - diff );
		}
	}

	int	n;
	n = this -> serv_io -> send( str );

#if 0
	cerr << "send[" << str << "]" << endl;
#endif

	this -> send_time.set_current_time();

	// EEE
	// compare int and size_t
	//
	if ( static_cast<size_t>(n) == str.length() )
	{
		return( n );
	}
	else
	{
		return( -1 );
	}
}


string  SServer_Connection::recv( bool  block )
{
	string	str;

	if ( block )
	{
		if ( serv_io -> blocking_ok() )
		{
			(void)(this -> serv_io -> recv( &str , block ));
		}
		else
		{
			// XXX
			(void)(this -> serv_io -> recv( &str , block ));
		}
	}
	else
	{
		return( this -> recv( static_cast<long>(0) ) );
	}

#if 0
	if ( str.length() != 0 )
	{
		cerr << "recv[" << str << "]" << endl;
	}
#endif
	return( str );
}


string  SServer_Connection::recv( long  usec )
{
	string	str;

	if ( serv_io -> non_blocking_ok() )
	{
		(void)(this -> serv_io -> recv( &str , usec ));
	}
	else
	{
		// XXX
		(void)(this -> serv_io -> recv( &str , usec ));
	}

#if 0
	// TTT
	if ( str.length() != 0 )
	{
		cerr << "recv[" << str << "]" << endl;
	}
#endif
	return( str );
}


int    SServer_Connection::send_init( const string &  team_name ,
				      bool  goalie ,
				      Game_Info *  g_info ,
				      int  major_version ,
				      int  minor_version )
{
	string	send_string = SServer_Message_Translator::init_command(
							      team_name ,
							      goalie ,
							      major_version ,
							      minor_version );


	int	ret = this -> send( send_string );


	string	buf;
	for(;;)
	{
		(void)(this -> serv_io -> recv( &buf , true , true ));
		if ( buf.length() != 0 )
		{
			break;
		}
	}

#if 0
	cout << "init_string = [" << buf << "]" << endl;
#endif

	Game_Info	g;
	SServer_Message_Translator::parse_init_message( &g , buf );

	this ->  team_name = team_name;
	this ->  our_side  = g.our_side;

	if ( g_info != NULL )
	{
		*g_info = g;
	}

	return( ret );
}

int    SServer_Connection::send_init( const string &  team_name ,
				      bool  goalie ,
				      Game_Info *  g_info )
{
	return( this -> send_init( team_name ,
				   goalie ,
				   g_info ,
				   SSERVER_DEFAULT_MAJOR_VERSION ,
				   SSERVER_DEFAULT_MINOR_VERSION ) );
}



int    SServer_Connection::send_reconnect( const string &  team_name ,
					   int  player_number ,
					   Game_Info *  g_info ,
					   long  interval_usec )
{
	string	send_string;
	send_string = SServer_Message_Translator::reconnect_command(
							    team_name ,
							    player_number );

	int	ret = this -> send( send_string , interval_usec );

	string	buf;
	for(;;)
	{
		(void)(this -> serv_io -> recv( &buf , true , true ));
		if ( buf.length() != 0 )
		{
			break;
		}
	}

#if 0
	cout << "recconect_string = [" << buf << "]" << endl;
#endif

	Game_Info	g;
	SServer_Message_Translator::
		parse_reconnect_message( &g , buf , player_number );

	this ->  team_name = team_name;
	this ->  our_side  = g.our_side;

	// XXX
	if ( g_info != NULL )
	{
		*g_info = g;
	}

	return( ret );
}


int    SServer_Connection::send_bye( long  interval_usec )
{
	string	send_string;
	send_string = SServer_Message_Translator::bye_command();

	return( this -> send( send_string , interval_usec ) );
}


int    SServer_Connection::send_move( double  x ,  double  y ,
				      long  interval_usec )
{
	string	send_string;
	send_string = SServer_Message_Translator::move_command( x , y );

	return( this -> send( send_string , interval_usec ) );
}


int    SServer_Connection::send_dash( double  power , long  interval_usec )
{
	string	send_string;
	send_string = SServer_Message_Translator::dash_command( power );

	return( this -> send( send_string , interval_usec ) );
}


int    SServer_Connection::send_kick( double  power ,  double  direction ,
				      long  interval_usec )
{
	string	send_string;
	send_string = SServer_Message_Translator::kick_command(
			power ,
			normalize_angle( direction , use_send_radian ) ,
			use_send_radian );

	return( this -> send( send_string , interval_usec ) );
}


int    SServer_Connection::send_turn( double  moment ,  long  interval_usec )
{
	string	send_string;
	send_string = SServer_Message_Translator::turn_command(
			normalize_angle( moment , use_send_radian ) ,
			use_send_radian );

	return( this -> send( send_string , interval_usec ) );
}


int    SServer_Connection::send_say( const string &  message ,
				     long  interval_usec )
{
	string	send_string;
	send_string = SServer_Message_Translator::say_command( message );

	return( this -> send( send_string , interval_usec ) );
}


int    SServer_Connection::send_catch( double  direction ,
				       long  interval_usec )
{
	string	send_string;
	send_string = SServer_Message_Translator::catch_command(
			normalize_angle( direction , use_send_radian ) ,
			use_send_radian );

	return( this -> send( send_string , interval_usec ) );
}


int    SServer_Connection::send_change_view( View_Angle  angle ,
					     View_Quality  quality ,
					     long  interval_usec )
{
	string	send_string;
	send_string = SServer_Message_Translator::change_view_command(
								angle ,
								quality );

	return( this -> send( send_string , interval_usec ) );
}


int    SServer_Connection::send_sense_body( long  interval_usec )
{
	string	send_string;
	send_string = SServer_Message_Translator::sense_body_command();

	return( this -> send( send_string , interval_usec ) );
}


int    SServer_Connection::send_turn_neck( double  direction ,
					   long  interval_usec )
{
	string	send_string;
	send_string = SServer_Message_Translator::turn_neck_command(
			    normalize_angle( direction , use_send_radian ) ,
			    use_send_radian );

	return( this -> send( send_string , interval_usec ) );
}


int    SServer_Connection::send_command( const Soccer_Command &  com ,
					 long  interval_usec )
{
	int	ret = -1;

	switch( com.type() )
	{
	case Soccer_Command::Move:
		ret = send_move( com.move_x() , com.move_y() , interval_usec );
		break;
	case Soccer_Command::Dash:
		ret = send_dash( com.dash_power() , interval_usec );
		break;
	case Soccer_Command::Kick:
		ret = send_kick( com.kick_power() , com.kick_direction() ,
				 interval_usec );
		break;
	case Soccer_Command::Turn:
		ret = send_turn( com.turn_moment() , interval_usec );
		break;
	case Soccer_Command::Catch:
		ret = send_catch( com.catch_direction() , interval_usec );
		break;
	case Soccer_Command::Turn_Neck:
		ret = send_turn_neck( com.turn_neck_direction() ,
				      interval_usec );
		break;
	case Soccer_Command::Change_View:
		ret = send_change_view( com.change_view_view_angle() ,
					com.change_view_view_quality() ,
					interval_usec );
		break;
	case Soccer_Command::Sense_Body:
		ret = send_sense_body( interval_usec );
		break;
	case Soccer_Command::Say:
		ret = send_say( com.say_message() , interval_usec );
		break;
	case Soccer_Command::Wait:
	case Soccer_Command::No_Command:
	case Soccer_Command::Illegal:
		break;
	}

	return( ret );
}


#if 0
//
// coach commands
//
int    SServer_Connection::send_look( long  interval_usec = 0 )
{
	string	send_string;
	send_string = SServer_Message_Translator::look_command();

	return( this -> send( send_string , interval_usec ) );
}

int    SServer_Connection::send_check_ball( long  interval_usec = 0 )
{
	string	send_string;
	send_string = SServer_Message_Translator::check_ball_command();

	return( this -> send( send_string , interval_usec ) );
}

int    SServer_Connection::send_ear( bool  on_off ,  long  interval_usec = 0 )
{
	string	send_string;
	send_string = SServer_Message_Translator::err_command( on_off );

	return( this -> send( send_string , interval_usec ) );
}

int    SServer_Connection::send_change_mode( Play_Mode  mode ,
					     long  interval_usec = 0 )
{
	string	send_string;
	send_string = SServer_Message_Translator::change_mode_command( mode );

	return( this -> send( send_string , interval_usec ) );
}
#endif




int    SServer_Connection::send_reconnect( const string &  team_name ,
					   int  player_number ,
					   Game_Info *  g_info ,
					   bool  default_wait )
{
	return( send_reconnect( team_name , player_number , g_info ,
				(default_wait ? default_wait_time : 0) ) );
}

int    SServer_Connection::send_bye( bool  default_wait )
{
	return( send_bye( (default_wait ? default_wait_time : 0) ) );
}

int    SServer_Connection::send_move( double  x ,  double  y ,
				      bool  default_wait )
{
	return( send_move( x , y , (default_wait ? default_wait_time : 0) ) );
}

int    SServer_Connection::send_dash( double  power ,  bool  default_wait )
{
	return( send_dash( power , (default_wait ? default_wait_time : 0) ) );
}

int    SServer_Connection::send_kick( double  power ,  double  direction ,
				      bool  default_wait )
{
	return( send_kick( power , direction ,
			   (default_wait ? default_wait_time : 0) ) );
}

int    SServer_Connection::send_turn( double  moment ,  bool  default_wait )
{
	return( send_turn( moment , (default_wait ? default_wait_time : 0) ) );
}

int    SServer_Connection::send_say( const string &  message ,
				     bool  default_wait )
{
	return( send_say( message , (default_wait ? default_wait_time : 0) ) );
}

int    SServer_Connection::send_catch( double  direction ,
				       bool  default_wait )
{
	return( send_catch( direction , (default_wait ? default_wait_time : 0) ) );
}

int    SServer_Connection::send_change_view( View_Angle  angle ,
					     View_Quality  quality ,
					     bool  default_wait )
{
	return( send_change_view( angle , quality ,
				  (default_wait ? default_wait_time : 0) ) );
}

int    SServer_Connection::send_sense_body( bool  default_wait )
{
	return( send_sense_body( (default_wait ? default_wait_time : 0) ) );
}

int    SServer_Connection::send_turn_neck( double  direction ,
					   bool  default_wait )
{
	return( send_turn_neck( direction ,
				(default_wait ? default_wait_time : 0) ) );
}

int    SServer_Connection::send_command( const Soccer_Command &  command ,
					 bool  default_wait )
{
	return( send_command( command ,
			      (default_wait ? default_wait_time : 0) ) );
}




Information_from_Server  SServer_Connection::recv_info( bool  block )
{
	Information_from_Server	info;

	string			mes = this -> recv( block );

	SServer_Message_Translator::parse_message( &info ,
						   mes ,
						   our_side ,
						   team_name ,
						   use_recv_radian );

	if ( info.type() == Information_from_Server::Type_Unknown )
	{
		cerr << "Unknown format: [" << info.raw_message() << "]"
		     << endl;
	}

	return( info );
}


Information_from_Server  SServer_Connection::recv_info( long  usec )
{
	Information_from_Server	info;

	string			mes = this -> recv( usec );

	SServer_Message_Translator::parse_message( &info ,
						   mes ,
						   our_side ,
						   team_name ,
						   use_recv_radian );

	if ( info.type() == Information_from_Server::Type_Unknown )
	{
		cerr << "Unknown format: [" << info.raw_message() << "]"
		     << endl;
	}

	return( info );
}


Information_from_Server  SServer_Connection::recv_info( int  usec )
{
	return( this -> recv_info( static_cast<long>(usec) ) );
}
