#ifdef    SSERVER_IO_TEST
#include  "sserver_io.h"

#include  <iostream>
#include  <stdexcept>
using namespace std;

extern "C"
{
#include  <unistd.h>
}



int    main( void )
{
	SServer_Socket	soc( "localhost" , 6000 );

	soc.send( "(init aaa)" );
	sleep( 1 );

	cout << "[" << soc.recv( true ) << "]" << endl;
	sleep( 1 );

	soc.send( "(move -10 0)" );
	sleep( 1 );

	soc.send( "(bye)" );

	return( 0 );
}
#endif // SSERVER_IO_TEST




//
// Implement
//
#include  "sserver_io.h"
#include  "ref_count_ptr.h"
#include  <cstdio>

extern "C"
{
	#include  <stdio.h>
	#include  <string.h>
	#include  <errno.h>
	#include  <sys/types.h>
	#include  <sys/socket.h>
	#include  <netinet/in.h>
	#include  <netdb.h>

	#ifdef HAVE_CONFIG_H
	    #if HAVE_UNISTD_H
		#include  <unistd.h>
	    #endif
	#else
	    #include  <unistd.h>
	#endif

	#ifdef HAVE_SYS_IOCTL_H
	    #if HAVE_SYS_IOCTL_H
		#include  <sys/ioctl.h>
	    #endif
	#else
	    #include  <sys/ioctl.h>
	#endif

	#ifdef HAVE_FCNTL_H
	    #if HAVE_FCNTL_H
		#include  <fcntl.h>
	    #endif
	#else
	    #include  <fcntl.h>
	#endif

	#if defined(TIME_WITH_SYS_TIME) && defined(HAVE_SYS_TIME_H)
	    #if TIME_WITH_SYS_TIME
		#include  <sys/time.h>
		#include  <time.h>
	    #else
		#if HAVE_SYS_TIME_H
		    #include  <sys/time.h>
		#else
		    #include  <time.h>
		#endif
	    #endif
	#else
	  #include  <sys/time.h>
	#endif
}

using namespace std;

SServer_Socket::SServer_Socket( const string &  host ,  in_port_t  port ,
				long  max_resp )
	: recv_time( 0 , 0 ) , max_response( max_resp )
{
	memset( &serv_addr , 0 , sizeof( serv_addr ) );

	if ( (sfd = ::socket( AF_INET , SOCK_DGRAM  , 0 ))  ==  -1 )
	{
		perror( "socket" );
		return;
	}

	struct sockaddr_in	my_addr;
	my_addr.sin_family      = AF_INET;
	my_addr.sin_addr.s_addr = htonl( INADDR_ANY );
	my_addr.sin_port        = htons( 0 );

	if ( ::bind( sfd , (const struct sockaddr *)(&my_addr) ,
			   sizeof( my_addr ) )  ==  -1 )
	{
		perror( "bind" );
		this -> close();
	}

	cout << "host = [" << host << "]" << endl;

	struct hostent *	hp;
	if ( (hp = ::gethostbyname( host.c_str() ))  ==  NULL )
	{
		perror( "gethostbyname" );
		this -> close();
		return;
	}

	serv_addr.sin_family = AF_INET;
	memcpy( &serv_addr.sin_addr.s_addr , hp -> h_addr , hp -> h_length );
	cout << "port = " << port << endl;
	serv_addr.sin_port = htons( port );
}


SServer_Socket::~SServer_Socket()
{
	this -> close();
}



SServer_Socket::operator bool()
{
	return( this -> fd() != -1 );
}

bool   SServer_Socket::respond()
{
	if ( ! (this -> operator bool()) )
	{
		return( false );
	}

	if ( this -> max_response == 0 )
	{
		return( true );
	}

	Time_Stamp	t;
	t.set_current_time();

	return( (t - recv_time).usec() < (this -> max_response) );
}


int    SServer_Socket::fd()
{
	return( sfd );
}


int    SServer_Socket::close()
{
	// if already closed or not opened
	if ( sfd  ==  -1 )
	{
		// return with no error
		return( 0 );
	}
	else
	{
		// close the file descriptor
		int	ret;

		if ( (ret = ::close( sfd ))  ==  -1 )
		{
			perror( "close" );
		}

		sfd = -1;

		return( ret );
	}
}

#if 0
in_port_t  SServer_Socket::port()
{
	return( port_num );
}

IP_Address  SServer_Socket::address()
{
}
#endif


bool   SServer_Socket::blocking_ok()
{
	return( true );
}

bool   SServer_Socket::non_blocking_ok()
{
	return( true );
}


int    SServer_Socket::send( const string &  str )
{
#if 1
	cout << "send [" << str << "]" << endl;
#endif

	int	n;
	if ( (n = ::sendto( sfd , str.c_str() , str.length() , 0 ,
			    (struct sockaddr *)(&serv_addr) ,
			    sizeof( serv_addr ) )) == -1 )
	{
		perror( "sendto" );
		return( -1 );
	}

	// EEE
	// compare int and size_t
	// It may cause problem.
	//
	if ( static_cast<size_t>(n) == str.length() )
	{
		return( n );
	}
	else
	{
		return( -1 );
	}
}


int    SServer_Socket::recv( string *  str ,  long  usec ,  bool  reply )
{
	// XXX
	ref_count_ptr<char>	buf;
	try
	{
		buf = new char[ SSERVER_MAX_MESSAGE_LENGTH + 1 ];
	}
	catch( ... )
	{
		cerr << "Can't allocate memory" << endl;
		throw;
	}

	struct sockaddr_in	from;
#ifdef __linux
	socklen_t		from_size;
#else
	int			from_size;
#endif
	int			n;
	int			ret;

	from_size = sizeof( from );

	struct timeval	tv;
	fd_set		readfd;
	FD_ZERO( &readfd );
	FD_SET( this -> sfd , &readfd );

	tv.tv_sec = usec / 1000000;
	tv.tv_usec = usec % 1000000;

	if ( ::select( this -> sfd + 1 , &readfd , NULL , NULL , &tv ) == 1 )
	{
		if ( (ret = ::recvfrom( this -> sfd ,
					buf.get() ,
					SSERVER_MAX_MESSAGE_LENGTH ,
					0 ,
					(struct sockaddr *)(&from) ,
					&from_size )) == -1 )
		{
			perror( "recvfrom" );
			n = 0;
		}
		else
		{
			n = ret;
			recv_time.set_current_time();
		}
	}
	else
	{
		ret = -1;
		n = 0;
	}

	buf.get()[n] = '\0';

	*str = buf.get();

	if ( reply )
	{
		if ( from.sin_port != static_cast<in_port_t>(0) )
		{
			serv_addr.sin_port = from.sin_port;
			cout << "port = "
			     << htons( serv_addr.sin_port ) << endl;
		}
	}

	return( ret );
}


int    SServer_Socket::recv( string *  str ,  bool  block ,  bool  reply )
{
	if ( this -> max_response != 0 )
	{
		return( this -> recv( str ,
				      (block ? (this -> max_response) : 0L) ,
				      reply ) );
	}

	// XXX
	ref_count_ptr<char>	buf;
	try
	{
		buf = new char[ SSERVER_MAX_MESSAGE_LENGTH + 1 ];
	}
	catch( ... )
	{
		cerr << "Can't allocate memory" << endl;
		throw;
	}

	struct sockaddr_in	from;
#ifdef __linux
	socklen_t		from_size;
#else
	int			from_size;
#endif
	int			n = 0;
	int			ret = 0;

	from_size = sizeof( from );

	if ( block )
	{
		if ( (ret = ::recvfrom( this -> sfd ,
					buf.get() ,
					SSERVER_MAX_MESSAGE_LENGTH ,
					0 ,
					(struct sockaddr *)(&from) ,
					&from_size )) == -1 )
		{
			if ( errno != EWOULDBLOCK )
			{
				perror( "recvfrom" );
			}
			n = 0;
		}
		else
		{
			n = ret;
			recv_time.set_current_time();
		}
	}

	buf.get()[n] = '\0';

	*str = buf.get();

	if ( reply )
	{
		if ( from.sin_port != static_cast<in_port_t>(0) )
		{
			serv_addr.sin_port = from.sin_port;
			cout << "port = "
			     << htons( serv_addr.sin_port ) << endl;
		}
	}

	return( ret );
}




//
//  SServer_IO_Stream
//
SServer_IO_Stream::SServer_IO_Stream( istream&  istrm , ostream&  ostrm )
	: in( istrm ) , out( ostrm )
{
}


SServer_IO_Stream::~SServer_IO_Stream()
{
}


SServer_IO_Stream::operator bool()
{
	return( true );
}

bool   SServer_IO_Stream::respond()
{
	return( this -> operator bool() );
}


int    SServer_IO_Stream::send( const string &  str )
{
	out << str << flush;

	return( str.length() );
}


int    SServer_IO_Stream::recv( string *  str ,  bool ,  bool )
{
	(*str) = "";

	char	ch;
	while( in.get( ch ) )
	{
		*str += ch;

		if ( ch == '\n' )
		{
			break;
		}
	}

	return( 0 );
}

int    SServer_IO_Stream::recv( string *  str ,  long ,  bool  reply )
{
	return( this -> recv( str , false , reply ) );
}


bool   SServer_IO_Stream::blocking_ok()
{
	return( true );
}

bool   SServer_IO_Stream::non_blocking_ok()
{
	return( false );
}
