#ifndef	   SSERVER_IO_H_INCLUDED
#define	   SSERVER_IO_H_INCLUDED

#ifdef HAVE_CONFIG_H
    #include  "config.h"
#endif

//
//  SServer_IO
//
#include  <string>
class  SServer_IO
{
public:
	static	const	size_t		SSERVER_MAX_MESSAGE_LENGTH = 2048;

public:
		 SServer_IO(){}
	virtual	~SServer_IO(){}

	virtual	operator bool() { return( false ); }
	virtual	bool	respond() = 0;

	virtual	int	send( const std::string &  str ) = 0;
	virtual	int	recv( std::string *  str ,  bool  block ,
			      bool  reply = false ) = 0;
	virtual	int	recv( std::string *  str ,  long  usec ,
			      bool  reply = false ) = 0;

	virtual	bool	blocking_ok()    { return( false ); }
	virtual	bool	non_blocking_ok(){ return( false ); }
};




//
//  SServer_Socket
//
#include  <string>
#include  "ip_address.h"
#include  "time_stamp.h"

class  SServer_Socket : public SServer_IO
{
public:
	static	const	in_port_t	SSERVER_DEFAULT_PORT = 6000;
	static	const	long		DEFAULT_MAX_RESPONCE_TIME
						= 3 * 1000 * 1000; // 3 sec

protected:
	int			sfd;
	struct sockaddr_in	serv_addr;

	Time_Stamp		recv_time;

	const long		max_response;

public:
		 SServer_Socket( const std::string &  host ,
				 in_port_t  port_num = SSERVER_DEFAULT_PORT ,
				 long  max_resp = DEFAULT_MAX_RESPONCE_TIME );
	virtual	~SServer_Socket();

	virtual	operator bool();
	virtual	bool	respond();

	virtual	int	send( const std::string &  str );
	virtual	int	recv( std::string *  str ,  bool  block ,
			      bool  reply = false );
	virtual	int	recv( std::string *  str ,  long  usec ,
			      bool  reply = false );

	virtual	bool	blocking_ok();
	virtual	bool	non_blocking_ok();

	virtual	int	fd();
	virtual	int	close();

#if 0
	virtual	in_port_t	port();
	virtual	IP_Address	address();
#endif
};




//
//  SServer_IO_Stream
//
#include  <string>
#include  <iostream>

class  SServer_IO_Stream : public SServer_IO
{
protected:
	istream&	in;
	ostream&	out;

public:
		 SServer_IO_Stream( istream&  istrm , ostream&  ostrm );
	virtual	~SServer_IO_Stream();

	virtual	operator bool();
	virtual	bool	respond();

	virtual	int	send( const std::string &  str );
	virtual	int	recv( std::string *  str ,  bool  block ,
			      bool  reply = false );
	virtual	int	recv( std::string *  str ,  long  usec ,
			      bool  reply = false );

	virtual	bool	blocking_ok();
	virtual	bool	non_blocking_ok();
};


#endif	/* SSERVER_IO_H_INCLUDED */
